library(genlasso)
library(pryr)
library(dplyr)
library(vegan)
library(rdist)
library(igraph)


### Calculate Variance of estimated coefficients by ESF-SVC
calc_Variance_esf <- function(X, beta, residuals, E)
{
  # Set id for coefficients
  id <- matrix(1:nrow(beta))
  beta <- as.data.frame(cbind(beta, id)) # add id for original beta
  colnames(beta) <- c("coef", "id")
  
  # position where beta2 related parameters starts and ends. (X consists of X1, X1*eigenvectors, X2, X2*eigenvectors, (Dummy(subregions))
  beta2_start_id <- ncol(E) + 2
  beta2_end_id <- ncol(E)*2 + 2
  
  beta <- dplyr::filter(beta, coef != 0) # Remove zero-coefficients from beta.
  nonzero_id <- beta$id # record id of beta with non-zero coefficients.
  id2 <- matrix(1:nrow(beta))  # Set id2 for non-zero coefficients
  beta <- as.data.frame(cbind(beta, id2)) # add id2 for beta
  colnames(beta) <- c("coef", "id", "id2")

  X_nonzero <- X[, nonzero_id] # Remove explanatory variables whose estimated coefficients are zero.
  XTX <- t(X_nonzero) %*% X_nonzero
  XTX_inv <- solve(XTX)
  
  df <- nrow(X_nonzero) - ncol(X_nonzero) # Degree of freedom
  residuals_sq <- as.numeric(t(residuals) %*% residuals)
  Var_matrix <- (residuals_sq / df) * XTX_inv # Variance-covariance matrix of non-zero coefficients

  # Calculate variance of estimated SVC
  beta1_related_coef <-
    beta[beta$id < beta2_start_id,] # Estimators of beta1 (X1, X1_ESF)
  beta1_nonzero_list <- as.matrix(beta1_related_coef$id2, ncol = 1) # set a list of id2 of beta1 (non-zero)
  
  beta2_related_coef <-
    beta[beta$id >= beta2_start_id & beta$id <= beta2_end_id,] # Estimators of beta2 (X2, X2_ESF)
  beta2_nonzero_list <- as.matrix(beta2_related_coef$id2, ncol = 1) # set a list of id2 of beta2 (non-zero)

  var_beta1 <- matrix(0, nrow = nrow(X_nonzero), ncol = 1)
  var_beta2 <- matrix(0, nrow = nrow(X_nonzero), ncol = 1)
  
  for (h in 1:nrow(X_nonzero)) {
    # Calculate variance of X * beta1_svc: ??_h Var(X_nonzero_h1 * beta_nonzero_1 + X_nonzero_h2 * beta_nonzero_2 + ...) 
    tmp <- 0
    for (i in 1:nrow(beta1_nonzero_list)) {
      for (j in 1:nrow(beta1_nonzero_list)){
        tmp <-
          tmp + X_nonzero[h, beta1_nonzero_list[i]] * X_nonzero[h, beta1_nonzero_list[j]] * Var_matrix[beta1_nonzero_list[i], beta1_nonzero_list[j]]
      }
    }
    # Calculate variance of beta1_svc (divide tmp by square of (X_1))
    var_beta1[h,] <- tmp / X_nonzero[h, beta1_nonzero_list[1]] / X_nonzero[h, beta1_nonzero_list[1]]
    
    # Calculate variance of X * beta2_svc: ??_h Var(X_nonzero_h1 * beta_nonzero_1 + X_nonzero_h2 * beta_nonzero_2 + ...) 
    tmp <- 0
    for (i in 1:nrow(beta2_nonzero_list)) {
      for (j in 1:nrow(beta2_nonzero_list)){
        tmp <-
          tmp + X_nonzero[h, beta2_nonzero_list[i]] * X_nonzero[h, beta2_nonzero_list[j]] * Var_matrix[beta2_nonzero_list[i], beta2_nonzero_list[j]]
      }
    }
    # Calculate variance of beta2_svc (divide tmp by square of (X_2))
    var_beta2[h, ] <- tmp / X_nonzero[h, beta2_nonzero_list[1]] / X_nonzero[h, beta2_nonzero_list[1]]
  }

  return(list(var_beta1, var_beta2))
}


### Calculate Variance of coefficients of non-SVC model
calc_Variance_nonSVC <- function(X, beta, residuals){
  # Set id for coefficients
  id <- matrix(1:nrow(beta))
  beta <- as.data.frame(cbind(beta, id))
  colnames(beta) <- c("coef", "id")

  beta <- dplyr::filter(beta, coef != 0) # Remove zero-coefficients.
  nonzero_id <- beta$id

  X_nonzero <-
    X[, nonzero_id] # Remove explanatory variables whose estimated coefficients are zero.
  XTX <- t(X_nonzero) %*% X_nonzero
  XTX_inv <- solve(XTX)
  
  df <- nrow(X_nonzero) - ncol(X_nonzero)
  residuals_sq <- as.numeric(t(residuals) %*% residuals)
  Var_matrix <-
    (residuals_sq / df) * XTX_inv # Variance-covariance matrix of coefficients
  
  var_beta1 <- matrix(Var_matrix[1,1], nrow = nrow(X_nonzero), ncol = 1)
  var_beta2 <- matrix(Var_matrix[2,2], nrow = nrow(X_nonzero), ncol = 1)
    
  return(list(var_beta1,
              var_beta2,
              Var_matrix))
}


### dualpathFusedL1X in genlasso package
dualpathFusedL1X <-
  function(y,
           X,
           D,
           D0,
           gamma,
           approx = FALSE,
           maxsteps = 2000,
           minlam = 0,
           rtol = 1e-7,
           btol = 1e-7,
           eps = 1e-4,
           verbose = FALSE,
           object = NULL) {
    # If we are starting a new path
    if (is.null(object)) {
      m = nrow(D)
      p = ncol(D)
      n = length(y)
      numedges = m - p
      numnodes = p

      # Modify y, X, n in the case of a ridge penalty, but
      # keep the originals
      y0 = y
      X0 = X
      if (eps > 0) {
        y = c(y, rep(0, p))
        X = rbind(X, diag(sqrt(eps), p))
        n = n + p
      }

      # Find the minimum 2-norm solution, using some linear algebra
      # tricks and a little bit of graph theory
      L = abs(crossprod(D0))
      diag(L) = 0
      gr = graph.adjacency(L, mode = "upper") # Underlying graph
      cl = clusters(gr)
      q = cl$no                            # Number of clusters
      i = cl$membership                    # Cluster membership

      # First we project y onto the row space of D*X^+
      xy = t(X) %*% y
      g = xy

      # Here we perform our usual fused lasso solve but
      # with g in place of y
      x = numeric(p)

      # For efficiency, don't loop over singletons
      tab = tabulate(i)
      oo = which(tab[i] == 1)
      if (length(oo) > 0) {
        x[oo] = g[oo] / (diag(L)[oo])
      }

      # Now all groups with at least two elements
      oi = order(i)
      cs = cumsum(tab)
      grps = which(tab > 1)
      for (j in grps) {
        oo = oi[Seq(cs[j] - tab[j] + 1, cs[j])]
        Lj = crossprod(Matrix(D[, oo], sparse = TRUE))
        x[oo] = as.numeric(solve(Lj, g[oo]))
      }

      uhat = as.numeric(D %*% x)   # Dual solution
      betahat = numeric(p)         # Primal solution
      ihit = which.max(abs(uhat))  # Hitting coordinate
      hit = abs(uhat[ihit])        # Critical lambda
      s = sign(uhat[ihit])         # Sign

      if (verbose) {
        cat(sprintf(
          "1. lambda=%.3f, adding coordinate %i, |B|=%i...",
          hit,
          ihit,
          1
        ))
      }

      # Now iteratively find the new dual solution, and
      # the next critical point

      # Things to keep track of, and return at the end
      buf = min(maxsteps, 1000)
      lams = numeric(buf)        # Critical lambdas
      h = logical(buf)           # Hit or leave?
      df = numeric(buf)          # Degrees of freedom

      lams[1] = hit
      h[1] = TRUE
      df[1] = 0

      u = matrix(0, m, buf)      # Dual solutions
      beta = matrix(0, p, buf)   # Primal solutions
      u[, 1] = uhat
      beta[, 1] = betahat

      # Special interior set over nodes
      I0 = rep(TRUE, numnodes)

      # Update the graph if we need to, otherwise
      # update the special interior set
      if (ihit <= numedges) {
        ed = which(D[ihit,] != 0)
        gr[ed[1], ed[2]] = 0             # Delete edge
        newcl = subcomponent(gr, ed[1])  # New cluster
        oldcl = which(i == i[ed[1]])     # Old cluster
        # If these two clusters aren't the same, update
        # the memberships
        if (length(newcl) != length(oldcl) ||
            any(sort(newcl) != sort(oldcl))) {
          i[newcl] = q + 1
          q = q + 1
        }
      }
      else {
        I0[ihit - numedges] = FALSE
      }

      # Other things to keep track of, but not return
      r = 1                      # Size of boundary set
      B = ihit                   # Boundary set
      I = Seq(1, m)[-ihit]        # Interior set
      D1 = D[-ihit, , drop = FALSE]  # Matrix D[I,]
      D2 = D[ihit, , drop = FALSE]   # Matrix D[B,]
      k = 2                      # What step are we at?
    }

    # If iterating already started path
    else {
      # Grab variables from outer object
      lambda = NULL
      for (j in 1:length(object)) {
        if (names(object)[j] != "pathobjs") {
          assign(names(object)[j], object[[j]])
        }
      }

      # Trick: save y,X from outer object
      y0 = y
      X0 = X

      # Grab variables from inner object
      for (j in 1:length(object$pathobjs)) {
        assign(names(object$pathobjs)[j], object$pathobjs[[j]])
      }
      lams = lambda

      # In the case of a ridge penalty, modify X
      if (eps > 0)
        X = rbind(X, diag(sqrt(eps), p))
    }

    tryCatch({
      while (k <= maxsteps && lams[k - 1] >= minlam) {
        ##########
        # Check if we've reached the end of the buffer
        if (k > length(lams)) {
          buf = length(lams)
          lams = c(lams, numeric(buf))
          h = c(h, logical(buf))
          df = c(df, numeric(buf))
          u = cbind(u, matrix(0, m, buf))
          beta = cbind(beta, matrix(0, p, buf))
        }

        ##########
        Ds = as.numeric(t(D2) %*% s)

        # Precomputation for the hitting times: first we project
        # y and Ds onto the row space of D1*X^+
        A = matrix(0, n, q)
        z = matrix(0, q, 2)
        nz = rep(FALSE, q)

        # For efficiency, don't loop over singletons
        tab = tabulate(i)
        oo = which(tab[i] == 1)
        oo2 = oo[!I0[oo]]
        if (length(oo2) > 0) {
          j = i[oo2]
          A[, j] = X[, oo2, drop = FALSE]
          z[j, 1] = xy[oo2]
          z[j, 2] = Ds[oo2]
          nz[j] = TRUE
        }

        # Now consider all groups with at least two elements
        grps = which(tab > 1)
        for (j in grps) {
          oo = which(i == j)
          if (all(!I0[oo])) {
            A[, j] = rowMeans(X[, oo, drop = FALSE])
            z[j, 1] = mean(xy[oo])
            z[j, 2] = mean(Ds[oo])
            nz[j] = TRUE
          }
        }

        nzq = sum(nz)
        e = matrix(0, q, 2)
        if (nzq > 0) {
          R = qr.R(qr(A[, nz]))
          e[nz,] = backsolve(R, forwardsolve(R, z[nz, , drop = FALSE], upper.tri =
                                               TRUE, transpose = TRUE))
          # Note: using a QR here is preferable than simply calling
          # e[nz,] = solve(crossprod(A[,nz]),z[nz,,drop=FALSE]), for
          # numerical stablity. Plus, it's not really any slower
        }
        ea = e[, 1]
        eb = e[, 2]
        ga = xy - t(X) %*% (A %*% ea)
        gb = Ds - t(X) %*% (A %*% eb)

        # If the interior is empty, then nothing will hit
        if (r == m) {
          fa = ea[i]
          fb = eb[i]
          hit = 0
        }

        # Otherwise, find the next hitting time
        else {
          # Here we perform our usual fused lasso solve but
          # with ga in place of y and gb in place of Ds
          xa = xb = numeric(p)
          fa = fb = numeric(p)

          # For efficiency, don't loop over singletons
          oo = which(tab[i] == 1)
          fa[oo] = ea[i][oo]
          fb[oo] = eb[i][oo]
          oo1 = oo[I0[oo]]
          if (length(oo1) > 0) {
            Ldiag = diag(crossprod(Matrix(D1[, oo1], sparse = TRUE)))
            xa[oo1] = ga[oo1] / Ldiag
            xb[oo1] = gb[oo1] / Ldiag
          }

          # Now all groups with at least two elements
          oi = order(i)
          cs = cumsum(tab)
          grps = which(tab > 1)
          for (j in grps) {
            oo = oi[Seq(cs[j] - tab[j] + 1, cs[j])]
            fa[oo] = ea[j] / length(oo)
            fb[oo] = eb[j] / length(oo)
            gaj = ga[oo]
            gbj = gb[oo]

            if (any(I0[oo])) {
              Lj = crossprod(Matrix(D1[, oo], sparse = TRUE))
              xa[oo] = as.numeric(solve(Lj, gaj))
              xb[oo] = as.numeric(solve(Lj, gbj))
            }
            else {
              Lj = crossprod(Matrix(D1[, oo[-1]], sparse = TRUE))
              xa[oo][-1] = as.numeric(solve(Lj, (gaj - mean(gaj))[-1]))
              xb[oo][-1] = as.numeric(solve(Lj, (gbj - mean(gbj))[-1]))
            }
          }

          a = as.numeric(D1 %*% xa)
          b = as.numeric(D1 %*% xb)
          shits = Sign(a)
          hits = a / (b + shits)

          # Make sure none of the hitting times are larger
          # than the current lambda (precision issue)
          hits[hits > lams[k - 1] + btol] = 0
          hits[hits > lams[k - 1]] = lams[k - 1]

          ihit = which.max(hits)
          hit = hits[ihit]
          shit = shits[ihit]
        }

        ##########
        # If nothing is on the boundary, then nothing will leave
        # Also, skip this if we are in "approx" mode
        if (r == 0 || approx) {
          leave = 0
        }

        # Otherwise, find the next leaving time
        else {
          c = as.numeric(s * (D2 %*% fa))
          d = as.numeric(s * (D2 %*% fb))
          leaves = c / d

          # c must be negative
          leaves[c >= 0] = 0

          # Make sure none of the leaving times are larger
          # than the current lambda (precision issue)
          leaves[leaves > lams[k - 1] + btol] = 0
          leaves[leaves > lams[k - 1]] = lams[k - 1]

          ileave = which.max(leaves)
          leave = leaves[ileave]
        }

        ##########
        # Stop if the next critical point is negative
        if (hit <= 0 && leave <= 0)
          break

        # If a hitting time comes next
        if (hit > leave) {
          # Record the critical lambda and properties
          lams[k] = hit
          h[k] = TRUE
          df[k] = nzq
          uhat = numeric(m)
          uhat[B] = hit * s
          uhat[I] = a - hit * b
          betahat = fa - hit * fb

          # Update our graph if we need to, otherwise
          # update the special interior set
          if (I[ihit] <= numedges) {
            ed = which(D1[ihit,] != 0)
            gr[ed[1], ed[2]] = 0             # Delete edge
            newcl = subcomponent(gr, ed[1])  # New cluster
            oldcl = which(i == i[ed[1]])      # Old cluster
            # If these two clusters aren't the same, update
            # the memberships
            if (length(newcl) != length(oldcl) ||
                any(sort(newcl) != sort(oldcl))) {
              i[newcl] = q + 1
              q = q + 1
            }
          }
          else {
            I0[I[ihit] - numedges] = FALSE
          }

          # Update all other variables
          r = r + 1
          B = c(B, I[ihit])
          I = I[-ihit]
          s = c(s, shit)
          D2 = rbind(D2, D1[ihit,])
          D1 = D1[-ihit, , drop = FALSE]

          if (verbose) {
            cat(sprintf(
              "\n%i. lambda=%.3f, adding coordinate %i, |B|=%i...",
              k,
              hit,
              B[r],
              r
            ))
          }
        }

        # Otherwise a leaving time comes next
        else {
          # Record the critical lambda and properties
          lams[k] = leave
          h[k] = FALSE
          df[k] = nzq
          uhat = numeric(m)
          uhat[B] = leave * s
          uhat[I] = a - leave * b
          betahat = fa - leave * fb

          # Update our graph if we need to, otherwise
          # update the special interior set
          if (B[ileave] <= numedges) {
            ed = which(D2[ileave,] != 0)
            gr[ed[1], ed[2]] = 1             # Add edge
            newcl = subcomponent(gr, ed[1])  # New cluster
            oldcl = which(i == i[ed[1]])      # Old cluster
            # If these two clusters aren't the same, update
            # the memberships
            if (length(newcl) != length(oldcl) ||
                !all(sort(newcl) == sort(oldcl))) {
              newno = i[ed[2]]
              oldno = i[ed[1]]
              i[oldcl] = newno
              i[i > oldno] = i[i > oldno] - 1
              q = q - 1
            }
          }
          else {
            I0[B[ileave] - numedges] = TRUE
          }

          # Update all other variables
          r = r - 1
          I = c(I, B[ileave])
          B = B[-ileave]
          s = s[-ileave]
          D1 = rbind(D1, D2[ileave,])
          D2 = D2[-ileave, , drop = FALSE]

          if (verbose) {
            cat(
              sprintf(
                "\n%i. lambda=%.3f, deleting coordinate %i, |B|=%i...",
                k,
                leave,
                I[m - r],
                r
              )
            )
          }
        }

        u[, k] = uhat
        beta[, k] = betahat

        # Step counter
        k = k + 1
      }
    }, error = function(err) {
      err$message = paste(
        err$message,
        "\n(Path computation has been terminated;",
        " partial path is being returned.)",
        sep = ""
      )
      warning(err)
    })

    # Trim
    lams = lams[Seq(1, k - 1)]
    h = h[Seq(1, k - 1)]
    df = df[Seq(1, k - 1)]
    u = u[, Seq(1, k - 1), drop = FALSE]
    beta = beta[, Seq(1, k - 1), drop = FALSE]

    # If we reached the maximum number of steps
    if (k > maxsteps) {
      if (verbose) {
        cat(sprintf("\nReached the max number of steps (%i),", maxsteps))
        cat(" skipping the rest of the path.")
      }
      completepath = FALSE
    }

    # If we reached the minimum lambda
    else if (lams[k - 1] < minlam) {
      if (verbose) {
        cat(sprintf("\nReached the min lambda (%.3f),", minlam))
        cat(" skipping the rest of the path.")
      }
      completepath = FALSE
    }

    # Otherwise, note that we completed the path
    else
      completepath = TRUE

    # The least squares solution (lambda=0)
    bls = NULL
    if (completepath)
      bls = fa
    if (verbose)
      cat("\n")

    # Save needed elements for continuing the path
    pathobjs = list(
      type = "fused.l1.x" ,
      r = r,
      B = B,
      I = I,
      Q1 = NA,
      approx = approx,
      Q2 = NA,
      k = k,
      df = df,
      D1 = D1,
      D2 = D2,
      Ds = Ds,
      ihit = ihit,
      m = m,
      n = n,
      p = p,
      q = q,
      h = h,
      q0 = NA,
      rtol = rtol,
      btol = btol,
      eps = eps,
      s = s,
      y = y,
      gr = gr,
      i = i,
      numedges = numedges,
      I0 = I0,
      xy = xy
    )

    colnames(u) = as.character(round(lams, 3))
    colnames(beta) = as.character(round(lams, 3))
    return(
      list(
        lambda = lams,
        beta = beta,
        fit = X0 %*% beta,
        u = u,
        hit = h,
        df = df,
        y = y0,
        X = X0,
        completepath = completepath,
        bls = bls,
        gamma = gamma,
        pathobjs = pathobjs
      )
    )
  }

Seq <- function(a, b, ...) {
  if (a <= b)
    return(seq(a, b, ...))
  else
    return(numeric(0))
}

Sign <- function(x) {
  return(-1 + 2 * (x >= 0))
}


### Calculate coefficient of determination and adjusted R-square
calc_R2 <- function(y, residuals, df) {
  mean_y <- matrix(mean(y), ncol = 1, nrow = nrow(y))
  n <- nrow(y)
  sq_difference <- (y - mean_y) ^ 2
  denominator <- sum(sq_difference)
  numerator <- sum(residuals * residuals)
  return(list(1 - numerator / denominator, 1 - (numerator / (n - df - 1)) / (denominator / (n - 1))))
}


### Calculate RMSE
calc_RMSE <- function(original, estimated) {
  rows <- nrow(original)
  RMSE <- sqrt((1 / rows) * sum((original - estimated) ^ 2))
  return(RMSE)
}


### Search the best estimation result based on BIC by OLS.
Search_best_model_by_OLS <- 
  function(y, X, D, D0, steps){
    # y: Dependent variables.
    # X: Explanatory variables (including variables calculated by eigenvectors).
    # D: Settings of weights on L1 regularization by sparse matrix (dgCMatrix).
    # D0: Settings of weights on L1 regularization.

    flsa = dualpathFusedL1X(
      y = y,
      X = X,
      D = D,
      D0 = D0,
      gamma = 1.0, # gamma is note effective in the function. The gamma should be represented by D and D0.
      approx = FALSE,
      maxsteps = steps,
      minlam = 0.001,
      rtol = 1e-7,
      btol = 1e-7,
      eps = 1e-4,
      verbose = FALSE,
      object = NULL
    )
    flsa$call = match.call()
    class(flsa) = c("fusedlasso", "genlasso", "list")
    
    summary.genlasso <- function(object) {
      n = length(object$y)
      p = nrow(object$beta)
      df = object$df
      y2 <- matrix(0, nrow = n, ncol = ncol(object$fit))
      object$y2 <- as.data.frame(y2)
      for (i in 1:ncol(object$y2)) {
        object$y2[, i] = y
      }
      
      rss = colSums((object$y2 - object$fit) ^ 2)
      
      mat = cbind(object$df, object$lambda, rss)
      rownames(mat) = rep("", nrow(mat))
      colnames(mat) = c("df", "lambda", "rss")
      
      class(mat) = c("summary.genlasso", "matrix")
      return(mat)
    }
    
    summary <- as.data.frame(summary.genlasso(object = flsa))
    group_df <- dplyr::group_by(summary, df)
    group_df <-
      dplyr::summarise(group_df,
                       min_lambda = min(lambda),
                       min_rss = min(rss))
    n <- nrow(y)
    bic <-
      dplyr::mutate(group_df, BIC = n * (log(2 * pi * min_rss / n) + 1) + df *
                      log(n)) # BIC calculated from estimates by lasso
    
    # Prepare a column to input BIC by OLS and number of effective coef, and ID.
    OLS_bic <- bic$BIC
    effective_coef <- bic$BIC
    bic_id <- matrix(1:nrow(bic), nrow = nrow(bic), ncol = 1)
    bic <- cbind(bic, OLS_bic, effective_coef, bic_id)
    
    temp_OLS_bic <- matrix(1e+100, nrow = nrow(bic), ncol = 1)
    temp_effective_coef <- matrix(0, nrow = nrow(bic), ncol = 1)
    
    # Recalculate BIC by OLS with explanatory variables whose coefficients are non-zero by fused lasso estimation
    # (starting from the model whose number of parameters is two)
    estimated_coef <- as.data.frame(matrix(0, nrow=ncol(X), ncol=1))
    
    for (j in 2:nrow(bic)) {
      temp_X <- X
      lambda <- bic[j, ]$min_lambda
      coef_lasso <- as.data.frame(as.matrix(coefficients(flsa, lambda = lambda)$beta, ncol = 1))
      
      # Set ID for coefficients
      id <- matrix(1:nrow(coef_lasso))
      coef_lasso <- cbind(id, coef_lasso)
      
      # Set ID for columns of explanatory matrix temp_X (=X_lasso)
      colnames(temp_X) <- id
      temp_X <- as.data.frame(temp_X)
      colnames(coef_lasso) <- c("id", "param")
      
      # remove zero-coefficients
      coef_non_zero <- dplyr::filter(coef_lasso, param != 0)
      
      # remove explanatory variables whose estimated coefficients are zero
      nonzeroid <- unique(coef_non_zero$id)
      X_non_zero <- temp_X[, nonzeroid]
      Trans_non_zero <- diag(nrow = ncol(X), ncol = ncol(X))[, nonzeroid]
      
      # id2: ID for columuns of X_non_zero and coef_non_zero
      id2 <- matrix(1:nrow(coef_non_zero))
      coef_non_zero <- cbind(id2, coef_non_zero)
      
      # Search coefficients whose values are the same (coef of adjacent subregion dummies)
      overlapped_id <- coef_non_zero %>% group_by(param) %>% filter(n() > 1)
      
      # If there are no coefficients with the same values
      if (nrow(overlapped_id) == 0){
        Trans_no_overlap <- Trans_non_zero
      }
      else{
        overlap_list <- as.matrix(unique(overlapped_id$param), ncol = 1)
        
        # Aggregate coefficients that have same values
        for (k in 1:nrow(overlap_list)) {
          # Record coefficients who have same value with overlap_list[k]
          overlapped_coef <-
            dplyr::filter(overlapped_id, param == overlap_list[k, ])
          
          # Extract columns of transformation matrix whose coefficient values are in overlap_list[k]
          # and calculate the sum of columns.
          Trans_col_non_zero_overlap <- Trans_non_zero[, overlapped_coef$id2]
          Trans_non_zero <- cbind(Trans_non_zero, rowSums(Trans_col_non_zero_overlap))
        }
        # remove explanatory variables of overlapped coef
        Trans_no_overlap <- Trans_non_zero[, -overlapped_id$id2]
      }
      
      Trans_no_overlap <- as.matrix(Trans_no_overlap)
      temp_X <- as.matrix(temp_X)
      X_no_overlap <- temp_X %*% Trans_no_overlap
      
      # Output BIC by OLS estimation
      OLS <- lm(y ~ X_no_overlap - 1)
      
      # Record BIC and # of effective coefficients
      temp_OLS_bic[j, ] <- BIC(OLS)
      temp_effective_coef[j, ] <- ncol(X_no_overlap)
      
      coef <- as.data.frame(Trans_no_overlap %*% as.matrix(OLS$coefficients, ncol = 1))
      estimated_coef <- cbind(estimated_coef, coef)
    }
    bic[, 5] <- temp_OLS_bic
    bic[, 6] <- temp_effective_coef
    
    min_bic <- min(as.matrix(bic %>% dplyr::select(OLS_bic)))
    best_lambda <- as.numeric(bic %>% dplyr::filter(OLS_bic == min_bic) %>% dplyr::select(min_lambda) %>% head(1))
    bic_id <- as.numeric(bic %>% dplyr::filter(OLS_bic == min_bic) %>% dplyr::select(bic_id) %>% head(1))
    
    best_coef <- as.matrix(estimated_coef[ ,bic_id])
    best_N_effective_coef <- bic[bic_id, 6]
    
    return(list(
      # lambda that outputs the best model (evaluated by BIC estimated by OLS)
      best_lambda,
      # Minimum BIC by OLS estimation
      min_bic,
      # Estimated coefficients of the best model
      best_coef,
      # Number of effective explanatory variables
      best_N_effective_coef
    ))  
  }


### Search lambda that output the minimum BIC by OLS
### Fusion model
ESFGLSVC_func <-
  function(y, X_ESF, X_dummy, D_adj_subregion, E, steps, gamma_eigenvector, gamma_subregion) {
    # Set explanatory variables including subregion dummies
    X_all <- cbind(X_ESF, X_dummy)
    
    # Set L1 constraint matrix
    # Add matrix of 0 elements on the left side of D_subregion_adj (info of subregions' adjacency)
    D_zero <-
      matrix(0, nrow = nrow(D_adj_subregion), ncol = ncol(X_ESF))
    
    # Set weight "gamma_eigenvector" on L1 regularization term for coefficients on explanatory * eigenvector except explanatory variables (1e-6).
    D_explanatory <-
      diag(gamma_eigenvector,
           nrow = ncol(X_all),
           ncol = ncol(X_all))

    # Set small weight on L1 regularization term for coefficient on explanatory variables
    n_coef <- floor(ncol(X_ESF) / (ncol(E)+1))
    
    for (i in 1:n_coef) {
      D_explanatory[(i - 1) * ncol(E) + 1, (i - 1) * ncol(E) + 1] <- 1e-6
    }
    
    # Set weight "gamma_subregion" on L1 regularization term for coefficients on subregion dummies.
    dum_begin <- ncol(X_ESF) + 1
    for (i in dum_begin:ncol(X_all)) {
      D_explanatory[i, i] <- gamma_subregion
    }
    
    D0 <- rbind(cbind(D_zero, D_adj_subregion), D_explanatory)
    D <- as(D0, "dgCMatrix")
    
    best_result <- Search_best_model_by_OLS(y, X_all, D, D0, steps)
    coef_all <- as.data.frame(best_result[[3]])

    coef_svc <- matrix(0, ncol = n_coef, nrow = nrow(E))
    
    # Number of parameters that correspond to one spatially varying coef.
    col_group <- ncol(E) + 1
    
    for (i in 1:n_coef) {
      # Non-spatially varying coef is located at the first column. 
      coef_constant <-
        matrix(coef_all[1 + (i - 1) * col_group, ], nrow = nrow(E), ncol = 1)

      # Coef of X*eigenvectors are located from second to last columns.
      coef_eigenvector <-
        coef_all[c((2 + (i - 1) * col_group):(i * col_group)), ]
      coef_eigenvector <- as.matrix(coef_eigenvector, ncol = 1)

      coef_svc[, i] <- coef_constant + E %*% coef_eigenvector
    }
    
    # Subregion-based coef.
    coef_subregion <-
      as.data.frame(coef_all[c((nrow(coef_all) - ncol(D_adj_subregion) + 1):nrow(coef_all)), ])

    return(list(
      # lambda when the best model (evaluated by BIC estimated by OLS) is estimated
      best_result[[1]],
      # BIC of OLS estimates of the best model
      best_result[[2]],
      # Estimated spatially varying coefficients of beta1 and beta2
      coef_svc,
      # Estimated coefficients of subregion dummies
      coef_subregion,
      # All estimated coefficients
      coef_all,
      # Number of effective explanatory variables
      best_result[[4]]
    ))
  }


#### ESF-SVC estimation ####
# X_ESF includes terms (explanatory variables * eigenvectors)
ESFSVC_func <-
  function(y, X_ESF, E, steps) {
    # Set weight 1.0 on L1 regularization term for coefficients on (explanatory variables * eigenvectors) .
    D0 <- diag(1.0,
               nrow = ncol(X_ESF),
               ncol = ncol(X_ESF))
    
    # Set small weight on L1 regularization term for coefficient on explanatory variables.
    for (i in 1:2) {
      D0[(i - 1) * ncol(E) + 1, (i - 1) * ncol(E) + 1] <- 1e-6
    }
    
    D <- as(D0, "dgCMatrix")
    
    # Set candidate models by lasso and select the best model based on BIC by OLS.
    best_result <- Search_best_model_by_OLS(y, X_ESF, D, D0, steps)
    coef_all <- as.data.frame(best_result[[3]])

    # Number of spatially varying coefficients
    n_coef <- floor(ncol(X_ESF) / ncol(E))
    coef_svc <- matrix(0, nrow = nrow(E), ncol = n_coef)

    # Number of parameters that correspond to one spatially varying coef.
    col_group <- ncol(E) + 1
    
    for (i in 1:n_coef) {
      # Non-spatially varying coef is located at the first column. 
      coef_constant <-
        matrix(coef_all[1 + (i - 1) * col_group,], nrow = nrow(E), ncol = 1)
      
      # Coef of X*eigenvectors are located from second to last columns.
      coef_eigenvector <-
        coef_all[c((2 + (i - 1) * col_group):(i * col_group)),]
      coef_eigenvector <- as.matrix(coef_eigenvector, ncol = 1)

      coef_svc[, i] <- coef_constant + E %*% coef_eigenvector
    }
    
    return(list(
      # lambda when the best model (evaluated by BIC estimated by OLS) is estimated
      best_result[[1]],
      # Minimum BIC
      best_result[[2]],
      # Estimated spatially varying coefficients of beta1 and beta2
      coef_svc,
      # Estimated coefficients
      coef_all,
      # Number of effective explanatory variables
      best_result[[4]]
    ))
  }


# Estimation of Fused LASSO
# D: Matrix that represents the constraints on subregion dummies (L1 penalty on themselves and difference between neighboring coefficients)
fusedlasso_func <- function(y, X, X_dummy, D_subregion, steps, gamma_dummy) {
  # Set explanatory variables including subregion dummies
  X_all <- cbind(X, X_dummy)
  
  # Set L1 constraint matrix
  # Add matrix of 0 elements on the left side of D_subregion
  D_zero <- matrix(0, nrow = nrow(D_subregion), ncol = ncol(X))

  # Set weights gammma_dummy on L1 regularization term for coefficients themselves
  D_coef <-
    diag(gamma_dummy,
         nrow = ncol(X_all),
         ncol = ncol(X_all))
  # Set small weight on L1 regularization term for coefficient on explanatory variables
  for (i in 1:2) {
    D_coef[i, i] <- 1e-6   
  }
  
  D0 <- rbind(cbind(D_zero, D_subregion), D_coef)
  D <- as(D0, "dgCMatrix")
  
  best_result <- Search_best_model_by_OLS(y, X_all, D, D0, steps)
  coef_all <- as.data.frame(best_result[[3]])
  coef_subregion <- coef_all[c((ncol(X) + 1):nrow(coef_all)), ]
  
  return(list(
    # lambda when the best model (evaluated by BIC estimated by OLS) is estimated
    best_result[[1]],
    # BIC of OLS estimates of the best model
    best_result[[2]],
    # alpha (coefficients of subregion dummies)
    coef_subregion,
    # All coefficients
    coef_all,
    # Number of effective explanatory variables
    best_result[[4]]
  ))
}


## Calculate Moran's I
calc_moran <- function(data, weightmat) {
  data <- as.matrix(data, ncol = 1)
  n <- nrow(data)
  mean_mat <- matrix(mean(data), nrow = n, ncol = 1)
  Vec <- data - mean_mat
  moransI <-
    n * (t(Vec) %*% (weightmat) %*% (Vec)) / (sum(weightmat) * (t(Vec) %*% (Vec)))
  return(moransI)
}


# Construction of spatial weight matrix
Construct_SpatialWeightMatrix <- function(bandwidth, coordinate) {
  D <- rdist(coordinate) # Distance matrix
  C <- exp(-(D / bandwidth) ^ 2) # Exponential
  return(list(as.matrix(D), as.data.frame(as.matrix(C))))
}


# Approximate calculation of eigenvectors by Murakami and Griffith (2019)
eigen_approx<- function(coords, bandwidth, knots, threshold_evec){
  n <- nrow(coords)
  enum <- knots # Number of knots
  h <- bandwidth
  coordk <- kmeans( coords, centers = enum, iter.max = 50 )$centers # Center coords of knots created by k-means
  D <- rdist( coordk ) # Distance matrix of knots
  C	<- exp( -( D / h) ^ 2 ) # Spatial weight matrix of knots
  C <- as.data.frame(as.matrix(C))
  diag(C) <- 0
  Cmean	  <- apply( C, 1, mean )
  MCM	    <- t( C - Cmean ) - Cmean + mean( Cmean )
  eigenC	<- eigen( MCM ) 
  sf0	    <- eigenC$vectors
  ev0	    <- eigenC$values
  L <- nrow(coordk)
  
  Dnl <- cdist( coords, coordk ) # Distance matrix between data and knots
  Cnl <- exp( -( Dnl / h) ^ 2 )  # Spatial weight matrix between data and knots
  one<-matrix(1, ncol=1, nrow=n) # n*1 vector of 1s
  one_l<-matrix(1, ncol=1, nrow=L) # L*1 vector of 1s
  identity<-diag(ncol=L, nrow=L) #L*L identity matrix
  eigen_lam<-diag(x=ev0, ncol=L, nrow=L) #??_L
  
  C_l<-as.matrix(C)
  
  block1<-(t(one_l)%*%C_l)/L 
  block2<- one %x% block1 
  block3<-Cnl-block2
  block4<-(identity+eigen_lam)
  block5<-block4
  for(i in 1:nrow(block5)){
    block5[i,i]<-1/block4[i,i]  
  }
  approx_eigen2<-block3%*%sf0%*%block5 
  
  lambda<-((L+n)/L)*(identity+eigen_lam)-identity
  
  evalue2<-diag(lambda)
  evalue2<-as.matrix(as.data.frame(evalue2))
  evalue3<-evalue2
  hiritsu<-matrix(0,ncol=1,nrow=nrow(evalue2))
  for(t in 1:nrow(hiritsu)){
    hiritsu[t,]<-evalue3[t,]/evalue3[1,]  
  }
  evalue3<-as.data.frame(cbind(evalue3,hiritsu))
  colnames(evalue3)<-c("V1","V2")
  evalue3<-evalue3[evalue3$V2 > threshold_evec,]
  
  approx_eigen<-approx_eigen2[,c(1:nrow(evalue3))] 

  # Standardize eigenvectors
  for(i in 1:length(evalue3)){
    absolute_value<-norm(approx_eigen2[,i], "2")
    approx_eigen[,i]<-approx_eigen2[,i]/absolute_value
  }
  
  return(list(approx_eigen, lambda, evalue3$V1)) 
}


# Set subregion id to data
Set_subregion_id<-function(coordinate, subregion_list){
  subregion_id<-matrix(0, nrow=n_data, ncol=1)
  for(k in 0:9){
    for(i in 0:9){
      for(j in 1:n_data){
        if(((k/10)<coordinate[j,1])&&(coordinate[j,1]<(k/10)+0.1)&&((i/10)<coordinate[j,2])&&(coordinate[j,2]<(i/10)+0.1)){
          subregion_id[j,]<-as.character(subregion_list[10*i+k+1,])
        }
      }
    }
  }
  return(subregion_id)
}


## Construct subregion dummy variables (reference subregion is not set yet)
# subregion_id: Subregion id of each data (The length of the vector is the size of data)
# subregion_list: List of Subregion id
Construct_subregion_dummy <- function(subregion_id, subregion_list) {
  D <- matrix(0,
              nrow = nrow(subregion_id),
              ncol = nrow(subregion_list))
  
  for (i in 1:nrow(subregion_id)) {
    for (j in 1:nrow(subregion_list)) {
      if (subregion_id[i, ] == subregion_list[j, ]) {
        D[i, j] <- 1
        break
      }
    }
  }
  return(D)
}


## Set dependent variables by adding disturbances
Add_distrubances <- function(model, sd_disturbance) {
  disturbance <- rnorm(nrow(model), 0, sd_disturbance)
  disturbance <- as.matrix(disturbance, ncol = 1)
  y <- model + disturbance
  return(y)
}


## Set reference subregion: Search subregion where sum of squares of residuals are the minimum
##                          and set it as reference subregion.
# sq_resid_point: Square of residual at each points.
# subregion_id: Subregion where each point is located in is recorded.
# subregion_list: list of subregion
set_reference_subregion <-
  function(sq_resid_point,
           subregion_id,
           subregion_dummy,
           subregion_list) {

    sum_sq_resid_subregion <- n_data_subregion <- matrix(0, nrow = nrow(subregion_list), ncol = 1)

    tmp_sum_sq_resid_subregion <- aggregate(x = sq_resid_point, by = list(subregion_id), sum)
    
    for (i in 1:nrow(subregion_list)) {
      n_data_subregion[i, ] <- sum(subregion_dummy[, i]) # count points in each subregion
      
      for (j in 1:nrow(subregion_list)) {
        if (subregion_list[i, ] == tmp_sum_sq_resid_subregion[j, 1]) {
          sum_sq_resid_subregion[i, ] <- tmp_sum_sq_resid_subregion[j, 2]
          break
        }
      }
    }
    
    RMSE_subregion <- sqrt(sum_sq_resid_subregion / n_data_subregion)
    min_RMSE_subregion <- which.min(RMSE_subregion)
    new_subregion_dummy <- subregion_dummy[, -min_RMSE_subregion]
    
    return(list(min_RMSE_subregion, new_subregion_dummy))
  }


# Construct contiguity matrix of 10 by 10 subregions
Construct_contiguity_matrix <-
  function(n_subregion, reference_subregion) {
    Contiguity <- matrix(0, ncol = n_subregion, nrow = n_subregion)
    for (k in 1:90) {
      Contiguity[k, k + 10] = Contiguity[k + 10, k] = 1
    }
    for (k in 1:99) {
      Contiguity[k, k + 1] = Contiguity[k + 1, k] = 1
    }
    
    k = c(10, 20, 30, 40, 50, 60, 70, 80, 90)
    Contiguity[k, k + 1] = 0
    Contiguity[k + 1, k] = 0
    
    Contiguity2 <- Contiguity[-reference_subregion, ]
    Contiguity3 <- Contiguity2[, -reference_subregion]
    
    # Construct contiguity graph from contiguity matrix
    Contiguity_graph <- graph.adjacency(Contiguity3, mode="upper")
    Sparse_congituity_matrix = getDgSparse(Contiguity_graph)
    
    return(Sparse_congituity_matrix)
  }


####### Estimate Fusion model #######
Estimate_ESFGLSVC_model <-
  function(y,
           X1,
           X2,
           E,
           coordinate,
           beta1_simulated,
           beta2_simulated,
           lambda_list,
           subregion_dummy,
           subregion_id,
           subregion_list) {
    ## Construct explanatory variables for ESF-SVC
    Xeigen1 <- matrix(0, nrow = nrow(E), ncol = ncol(E))
    Xeigen2 <- matrix(0, nrow = nrow(E), ncol = ncol(E))
    
    for (i in 1:ncol(E)) {
      Xeigen1[, i] <- X1 * E[, i]
      Xeigen2[, i] <- X2 * E[, i]
    }
    
    X_ESF <- cbind(X1, Xeigen1, X2, Xeigen2)
    
    #### Estimation by ESF-SVC ####
    # ESF-SVC with lasso constraints
    ESFSVC <-
      ESFSVC_func(
        y = y,
        X_ESF = X_ESF,
        E = E,
        steps = 1000
      ) 
    ESFSVC_coefs <- as.matrix(as.data.frame(ESFSVC[4]))
    ESFSVC_residuals <- y - X_ESF %*% ESFSVC_coefs
    ESFSVC_df <- as.numeric(ESFSVC[5])
    
    ## Calculate statistics
    # Variance
    ESFSVC_Variance <-
      calc_Variance_esf(
        X = X_ESF,
        beta = ESFSVC_coefs,
        residual = ESFSVC_residuals,
        E = E
      )
    
    var_beta1 <- var_beta2 <- matrix(0, nrow=nrow(X_ESF), ncol=1)
    
    aa <- as.data.frame(ESFSVC_Variance[1])
    for (mm in 1:nrow(aa)) {
      var_beta1[mm, ] <- aa[mm, ]
    }
    aa <- as.data.frame(ESFSVC_Variance[2])
    for (mm in 1:nrow(aa)) {
      var_beta2[mm, ] <- aa[mm, ]
    }

    # R^2
    ESFSCV_R2 <- calc_R2(y = y, residual = ESFSVC_residuals, df = ESFSVC_df)
    
    # Moran's I of residuals
    ESFSVC_moran_residual <-
      calc_moran(data = ESFSVC_residuals, weightmat = weightm)
    
    # RMSE between estimated and set coefficients
    ESFSVC_varycoefs <-
      cbind(coordinate,
            beta1_simulated,
            beta2_simulated,
            as.data.frame(ESFSVC[3]))
    colnames(ESFSVC_varycoefs) <-
      c("x",
        "y",
        "beta1_simulated",
        "beta2_simulated",
        "beta1_estimated",
        "beta2_estimated")
    ESFSVC_RMSE_beta1 <-
      calc_RMSE(original = beta1_simulated, estimated = ESFSVC_varycoefs$beta1_estimated)
    ESFSVC_RMSE_beta2 <-
      calc_RMSE(original = beta2_simulated, estimated = ESFSVC_varycoefs$beta2_estimated)
    ESFSVC_RMSE_y <-
      calc_RMSE(original = y, estimated = y - ESFSVC_residuals)
    
    ESFSVC_statistics <-
      cbind(
        as.numeric(ESFSVC[1]),
        as.numeric(ESFSVC[2]),
        as.numeric(ESFSVC_moran_residual),
        as.numeric(ESFSCV_R2[1]),
        as.numeric(ESFSCV_R2[2]),
        as.numeric(ESFSVC_RMSE_beta1),
        as.numeric(ESFSVC_RMSE_beta2),
        as.numeric(ESFSVC_RMSE_y),
        as.numeric(ESFSVC_df),
        as.numeric(mean(var_beta1)),
        as.numeric(mean(var_beta2))
      )
    rownames(ESFSVC_statistics) <- NULL
    colnames(ESFSVC_statistics) <-
      c(
        "lambda",
        "BIC",
        "Moran's_I_residuals",
        "R2",
        "R2_adjusted",
        "beta1_RMSE",
        "beta2_RMSE",
        "Dependent_RMSE",
        "Effective_coefs",
        "average_variance_beta1",
        "average_variance_beta2"
      )
    
    ESFSVC_SVCoefs <-
      cbind(
        coordinate,
        beta1_simulated,
        beta2_simulated,
        as.data.frame(ESFSVC[3]),
        as.data.frame(ESFSVC_Variance[1]),
        as.data.frame(ESFSVC_Variance[2])
      )
    colnames(ESFSVC_SVCoefs) <-
      c(
        "x",
        "y",
        "beta1_simulated",
        "beta2_simulated",
        "beta1_estimated",
        "beta2_estimated",
        "variance_beta1",
        "variance_beta2"
      )
    
    #### Estimation by fusion model (ESF-SVC + fused lasso)  ####
    ## Set subregion dummy
    # Set reference subregion and remove the row of reference subregion from dummy matrix
    ref_subregion_result <-
      set_reference_subregion(
        sq_resid_point = ESFSVC_residuals * ESFSVC_residuals,
        subregion_id = subregion_id,
        subregion_dummy = subregion_dummy,
        subregion_list = subregion_list
      )
    ref_subregion <- ref_subregion_result[[1]]
    Dummy <- ref_subregion_result[[2]]
    
    # Create sparse matrix D_lasso that represent constraints on absolute values of coefficients and absolute differences between coefficients with contiguity
    D_lasso <-
      Construct_contiguity_matrix(n_subregion = n_subregion, reference_subregion = ref_subregion)
    
    X_fusion <- cbind(X_ESF, Dummy)
    
    # Prepare matrices to keep the results for all hyperparameter settings
    ESFGLSVC_lambda <-
      ESFGLSVC_bic <- ESFGLSVC_moran_residual <- ESFGLSVC_R2 <- ESFGLSVC_R2adj <- ESFGLSVC_df <-
      ESFGLSVC_RMSE_beta1 <- ESFGLSVC_RMSE_beta2 <-
      ESFGLSVC_aveVar_beta1 <- ESFGLSVC_aveVar_beta2 <-
      ESFGLSVC_RMSE_y <- matrix(0, nrow = 1, ncol = ncol(lambda_list))
    ESFGLSVC_beta1_estimated <- ESFGLSVC_beta2_estimated <-
      ESFGLSVC_var_beta1 <- ESFGLSVC_var_beta2 <-
      ESFGLSVC_residual <- matrix(0, nrow = nrow(y), ncol = ncol(lambda_list))
    ESFGLSVC_dummy_estimated <-
      matrix(0, nrow = ncol(Dummy), ncol = ncol(lambda_list))
    
    ## Estimate fusion model under different hyperparameter settings
    for (pp in 1:ncol(lambda_list)) {
      gamma_eigen <- lambda_list[2, pp]
      gamma_dummy <- lambda_list[3, pp]
      
      # Estimation
      Fusion <-
        ESFGLSVC_func(
          y = y,
          X_ESF = X_ESF,
          X_dummy = Dummy,
          D_adj_subregion = D_lasso,
          E = E,
          steps = 1000,
          gamma_eigenvector = gamma_eigen,
          gamma_subregion = gamma_dummy
        )
      
      # Estimated results for one setting of hyperparameters-> Record all results and return the best one
      ESFGLSVC_lambda[pp] <- as.matrix(as.data.frame(Fusion[1]))
      ESFGLSVC_bic[pp] <- as.matrix(as.data.frame(Fusion[2]))
      
      # Record estimated result and its statistics for a hyperparameter setting
      varycoefs <- as.matrix(as.data.frame(Fusion[3]))
      ESFGLSVC_beta1_estimated[, pp] <- varycoefs[, 1]
      ESFGLSVC_beta2_estimated[, pp] <- varycoefs[, 2]
      ESFGLSVC_dummy_estimated[, pp] <- as.matrix(as.data.frame(Fusion[4]))
      
      allcoefs <- as.matrix(as.data.frame(Fusion[5]))
      residual <- y - X_fusion %*% allcoefs
      ESFGLSVC_residual[, pp] <- residual
      ESFGLSVC_df[pp] <- as.numeric(Fusion[6])
      
      ESFGLSVC_Variance <-
        calc_Variance_esf(
          X = X_fusion,
          beta = allcoefs,
          residual = residual,
          E = E
        )
      ESFGLSVC_variance <- sapply(ESFGLSVC_Variance, as.numeric)

      aa <- as.data.frame(ESFGLSVC_Variance[1])
      for (mm in 1:nrow(aa)) {
        ESFGLSVC_var_beta1[mm, pp] <- aa[mm, ]
      }
      ESFGLSVC_aveVar_beta1[pp] <- mean(ESFGLSVC_var_beta1[, pp])
      
      aa <- as.data.frame(ESFGLSVC_Variance[2])
      for (mm in 1:nrow(aa)) {
        ESFGLSVC_var_beta2[mm, pp] <- aa[mm, ]
      }
      ESFGLSVC_aveVar_beta2[pp] <- mean(ESFGLSVC_var_beta2[,pp])
      
      ESFGLSVC_moran_residual[pp] <-
        calc_moran(data = residual, weightmat = weightm)
      R2 <-
        as.matrix(as.data.frame(calc_R2(
          y = y,
          residual = residual,
          df =  ESFGLSVC_df[pp]
        )))
      ESFGLSVC_R2[pp] <- R2[, 1]
      ESFGLSVC_R2adj[pp] <- R2[, 2]
      
      ESFGLSVC_RMSE_beta1[pp] <-
        calc_RMSE(
          original = beta1_simulated,
          estimated = Dummy %*% as.matrix(as.data.frame(Fusion[4])) + varycoefs[, 1]
        )
      ESFGLSVC_RMSE_beta2[pp] <-
        calc_RMSE(original = beta2_simulated, estimated = varycoefs[, 2])
      ESFGLSVC_RMSE_y[pp] <- calc_RMSE(original = y, estimated = y - residual)
    }
    
    # Search the best hyperparameter setting based on BIC
    best <- which.min(ESFGLSVC_bic)
    
    ESFGLSVC_statistics <-
      cbind(
        lambda_list[2, best],
        lambda_list[3, best],
        ESFGLSVC_lambda[best],
        ESFGLSVC_bic[best],
        ESFGLSVC_moran_residual[best],
        ESFGLSVC_R2[best],
        ESFGLSVC_R2adj[best],
        ESFGLSVC_RMSE_beta1[best],
        ESFGLSVC_RMSE_beta2[best],
        ESFGLSVC_RMSE_y[best],
        ESFGLSVC_df[best],
        ESFGLSVC_aveVar_beta1[best],
        ESFGLSVC_aveVar_beta2[best]
      )
    colnames(ESFGLSVC_statistics) <-
      c(
        "gamma_eigenvalue",
        "gamma_dummy",
        "lambda",
        "BIC",
        "Moran's_I_residuals",
        "R2",
        "R2_adjusted",
        "beta1_RMSE",
        "beta2_RMSE",
        "Dependent_RMSE",
        "Effective_coefs",
        "average_variance_beta1",
        "average_variance_beta2"
      )
    rownames(ESFGLSVC_statistics) <- NULL
    
    ESFGLSVC_estimates_explanatory <-
      cbind(
        coordinate,
        beta1_simulated,
        beta2_simulated,
        ESFGLSVC_beta1_estimated[, best],
        ESFGLSVC_beta2_estimated[, best],
        ESFGLSVC_var_beta1[, best],
        ESFGLSVC_var_beta2[, best],
        Dummy %*% ESFGLSVC_dummy_estimated[, best]
      )
    colnames(ESFGLSVC_estimates_explanatory) <-
      c(
        "x",
        "y",
        "beta1_simulated",
        "beta2_simulated",
        "beta1_estimated",
        "beta2_estimated",
        "variance_beta1",
        "variance_beta2",
        "subregion_coef"
      )
    
    ESFGLSVC_estimates_subregion <-
      as.data.frame(ESFGLSVC_dummy_estimated[, best])
    ESFGLSVC_residuals <- as.data.frame(ESFGLSVC_residual[, best])

    return(list(
      list(
        ESFSVC_statistics,
        ESFSVC_SVCoefs,
        ESFSVC_coefs,
        ESFSVC_residuals
      ),
      list(
        ESFGLSVC_statistics,
        ESFGLSVC_estimates_explanatory,
        ESFGLSVC_estimates_subregion,
        ESFGLSVC_residuals
      )
    ))
  }


##################################
############## Main ##############
##################################

#### Settings for simulation data generation ####
# Bandwidth
bandwidth = 0.2

# Average number of data in a subregion
n_data_per_subregion = 20

# Threshold for eigenvectors
threshold_evec <- 0.25

# Subregion dummy setting: Set non-zero subregion coefficients
# Coefficients of subregions 45, 46, 55, and 56 are set to 1.
alpha_vary <- matrix(0, nrow = 100, ncol = 1)
alpha_vary[45, ] <-
  alpha_vary[55, ] <-
  alpha_vary[46, ] <-
  alpha_vary[56, ] <-
  1.0

# Standard deviation of disturbances
sd_disturbance <- 0.2

# Setting of subregions (10 by 10)
height <- 10
width <- 10
n_subregion <- height * width
n_data <- n_subregion * n_data_per_subregion
################################################

################################################
# Setting of Hyperparameters for fusion model
lambda_list<-matrix(c(1,1,1,1,1,10,1,10,1,1,0.1,0.1,1,10,10,1,1,0.1,1,0.1,1,1,0.1,10,1,10,0.1),nrow=3)
rownames(lambda_list)<-c("lambda1","lambda2","lambda3")

# Setting of Hyperparameters for fused lasso
fusedlasso_lambda <- matrix(0, ncol = 1, nrow = 3)
fusedlasso_lambda[1, ] <- 0.1
fusedlasso_lambda[2, ] <- 1
fusedlasso_lambda[3, ] <- 10

sd_Xbeta_svc1<-sd_Xbeta_svc2<-sd_dummy<-sd_list<-sd_model<-utilized_Eigenvectors<-NULL
ESFGLSVC_statistics <- NULL
################################################  

# Number of simulation
N_simulation <- 1

for(q in 1:N_simulation){
  # When average number of points in a subregion is larger than 5, points are randomly distributed in all study region.
  if (n_data_per_subregion>5){
    x_coord<-matrix(runif(n_data, min=0, max=1), ncol=1)
    y_coord<-matrix(runif(n_data, min=0, max=1), ncol=1)
    coordinate<-cbind(x_coord, y_coord)
  }else{    # When average number of points in a subregion is not larger than 5, points are randomly distributed in each subregion.
    coordinate2<-matrix(0, ncol=2, nrow=1)
    x_coord_list<-y_coord_list<-as.matrix(c(0,0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9),ncol=1)
    for(i in 1:10) { 
      for(j in 1:10){
        x_coord2<-matrix(runif(n_data_per_subregion,min = x_coord_list[i,], max=x_coord_list[i,]+0.1), ncol=1)
        y_coord2<-matrix(runif(n_data_per_subregion,min = y_coord_list[j,], max=y_coord_list[j,]+0.1), ncol=1)
        coordinate1<-cbind(x_coord2, y_coord2)
        coordinate2<-rbind(coordinate2, coordinate1)
      }
    }
    coordinate<-coordinate2[2:nrow(coordinate2),]
  }
  
  ## Construction of spatial weight matrix for Moran's I calculation
  DandC <- Construct_SpatialWeightMatrix(bandwidth = bandwidth, coordinate = coordinate)
  D<-as.matrix(DandC[1]) # Distance matrix
  weightm<-as.matrix(as.data.frame(DandC[2])) # Spatial weight matrix
  
  ## Calculate approximate eigenvectors
  evec_calc <-
    eigen_approx(
      coords = coordinate,
      bandwidth = bandwidth,
      knots = 100,
      threshold_evec = threshold_evec
    )
  # E: Matrix of eigenvectors whose eigenvalues are larger than threshold_evec
  E <- as.matrix(as.data.frame(evec_calc[1])) # Eigenvectors
  write.csv(E, paste("Trial_", q, "_Eigenvectors_larger_threshold.csv", sep=""))

  ## Generation of coefficients
  # n_data: Number of data
  # beta1_const, beta2_const: Non-SVC coefficients
  # nonzero_1/2: Non-zero eigenvectors for beta1 and beta2

  # Selection of eigenvectors and settings of their weights
  selected_E <- as.integer(runif(2, min = 1, max = ncol(E)+1))
  weight_selected_E <- runif(2, min=1, max=10)
  utilized_Eigenvectors <- rbind(utilized_Eigenvectors, cbind(ncol(E), t(selected_E), t(weight_selected_E)))

  colnames(utilized_Eigenvectors) <-
    c(
      "N_eigenvectors",
      "beta1_eigenvector",
      "beta2_eigenvector",
      "beta1_eigenvector_coef",
      "beta2_eigenvector_coef"
    )
  # Eigenvectors and weights for simulating SVC coefficients
  write.csv(utilized_Eigenvectors, "Utilized_Eigenvectors.csv")
  
  # Intercept: Select non-zero eigenvector
  X1 <- matrix(1, nrow=n_data, ncol=1) # Explanatory variable for beta1 (intercept)
  beta1_const <- matrix(1, nrow=n_data, ncol=1)
  beta1_e <- matrix(0, nrow = ncol(E), ncol = 1)
  # Set coefficients beta0_e1 for eigenvector[select_eigenvector[1]]
  beta1_e[selected_E[1], ] <- weight_selected_E[1]

  beta1_esf <- beta1_const + E %*% beta1_e # Set spatial varying intercept
  model_svc1 <- X1 * beta1_esf
  model_1 <- X1 * beta1_const
  
  ## Generation of coefficients of explanatory variables
  X2 <-
    matrix(runif(n = n_data, min = 0, max = 1),
           nrow = n_data,
           ncol = 1) # Explanatory variable for beta2
  beta2_const <- matrix(1, nrow = n_data, ncol = 1)
  
  # explanatory variable: Select non-zero eigenvector
  beta2_e <- matrix(0, nrow = ncol(E), ncol = 1)
  # Set coefficients beta1_e_1 for eigenvector[selected_E[2]]
  beta2_e[selected_E[2], ] <- weight_selected_E[2]

  beta2_esf <-
    beta2_const + E %*% beta2_e # Set spatial varying coefficient
  model_svc2 <- X2 * beta2_esf
  model_2 <- X2 * beta2_const
  
  # Set subregion id (character) for data
  subregion_list <- matrix(c(1:100), ncol = 1)
  subregion_list <- as.character(subregion_list)
  subregion_list <- matrix(subregion_list, ncol = 1)
  subregion_id <-
    Set_subregion_id(coordinate = coordinate, subregion_list = subregion_list) # Set subregion ID for every data
  subregion_dummy <-
    Construct_subregion_dummy(subregion_id = subregion_id, subregion_list = subregion_list)
  model_subregion <-
    subregion_dummy %*% alpha_vary # Set non-zero coefficients for subregions
  
  # Record standard deviation
  sd_Xbeta_svc1 <- rbind(sd_Xbeta_svc1, sd(model_svc1))
  sd_Xbeta_svc2 <- rbind(sd_Xbeta_svc2, sd(model_svc2))
  sd_dummy <- rbind(sd_dummy, sd(model_subregion))

  # Values explained by models
  model <-
    model_svc1 + model_svc2 + model_subregion # Simulated data with global and Local heterogeneity

  beta1 <- model_svc1 + model_subregion
  beta2 <- beta2_esf
  sd_model <- rbind(sd_model, sd(model))

  y <-
    Add_distrubances(model = model, sd_disturbance = sd_disturbance)

  # Analysis of data with global and local heterogeneity
  results <- Estimate_ESFGLSVC_model(y=y, X1=X1, X2=X2, E=E, coordinate=coordinate, 
                                 beta1_simulated=beta1, beta2_simulated=beta2,
                                 lambda_list=lambda_list, subregion_dummy=subregion_dummy, subregion_id=subregion_id, subregion_list=subregion_list)
  ESFGLSVC_statistics <- rbind(ESFGLSVC_statistics, as.data.frame(results[[2]][1]))

  # Output results
  ESFGLSVC_SVcoefficients <- as.data.frame(results[[2]][2])
  ESFGLSVC_subregion_coefficients <- as.data.frame(results[[2]][3])
  ESFGLSVC_residuals <- as.data.frame(results[[2]][4])

  # Output of results
  # Fusion model
  write.csv(ESFGLSVC_SVcoefficients, paste("Trial_", q, "_ESFGLSVC_SVcoefficients.csv", sep=""))
  write.csv(ESFGLSVC_subregion_coefficients, paste("Trial_", q, "_ESFGLSVC_subregion_coefficients.csv", sep=""))
  write.csv(ESFGLSVC_residuals, paste("Trial_", q, "_ESFGLSVC_residuals.csv", sep=""))

  # Statistics for input data
  Inputdata_variance <- cbind(sd_Xbeta_svc1, sd_Xbeta_svc2, sd_dummy, sd_model)
  colnames(Inputdata_variance) <-
    c(
      "X_betasvc1",
      "X_betasvc2",
      "Dummy",
      "model"
    )
  write.csv(Inputdata_variance, "sd_data.csv")
  
  # Output statistics of Fusion model
  write.csv(ESFGLSVC_statistics, "ESFGLSVC_statistics.csv")
}
