library(genlasso)
library(dplyr)
library(rdist)

### dualpathFusedL1X in "genlasso" package ###
# We compute a solution path of the sparse fused lasso dual problem:
#
# \hat{u}(\lambda) =
# \argmin_u \|y - (X^+)^T D^T u\|_2^2 \rm{s.t.} \|\u\|_\infty \leq \lambda
#
# where D is (a multiple of) incidence matrix of a given graph, row-
# binded with (a multiple of) the identity matrix, and X is a full column
# rank predictor matrix, X^+ being its pseudoinverse.
#
# Fortuitously, we never have to fully invert X (i.e. compute its pseudo-
# inverse).
#
# Note: the df estimates at each lambda_k can be thought of as the df
# for all solutions corresponding to lambda in (lambda_k,lambda_{k-1}),
# the open interval to the *right* of the current lambda_k.
dualpathFusedL1X <- function(y, X, D, D0, gamma, approx=FALSE, maxsteps=2000,
                             minlam=0, rtol=1e-7, btol=1e-7, eps=1e-4,
                             verbose=FALSE, object=NULL) {
  # If we are starting a new path
  if (is.null(object)) {
    m = nrow(D)
    p = ncol(D)
    n = length(y)
    numedges = m-p
    numnodes = p
    
    # Modify y,X,n in the case of a ridge penalty, but
    # keep the originals
    y0 = y
    X0 = X
    if (eps>0) {
      y = c(y,rep(0,p))
      X = rbind(X,diag(sqrt(eps),p))
      n = n+p
    }
    
    # Find the minimum 2-norm solution, using some linear algebra
    # tricks and a little bit of graph theory
    L = abs(crossprod(D0))
    diag(L) = 0
    gr = graph.adjacency(L,mode="upper") # Underlying graph
    cl = clusters(gr)
    q = cl$no                            # Number of clusters
    i = cl$membership                    # Cluster membership
    
    # First we project y onto the row space of D*X^+
    xy = t(X)%*%y
    g = xy
    
    # Here we perform our usual fused lasso solve but
    # with g in place of y
    x = numeric(p)
    
    # For efficiency, don't loop over singletons
    tab = tabulate(i)
    oo = which(tab[i]==1)
    if (length(oo)>0) {
      x[oo] = g[oo]/(diag(L)[oo])
    }
    
    # Now all groups with at least two elements
    oi = order(i)
    cs = cumsum(tab)
    grps = which(tab>1)
    for (j in grps) {
      oo = oi[Seq(cs[j]-tab[j]+1,cs[j])]
      Lj = crossprod(Matrix(D[,oo],sparse=TRUE))
      x[oo] = as.numeric(solve(Lj,g[oo]))
    }
    
    uhat = as.numeric(D%*%x)     # Dual solution
    betahat = numeric(p)         # Primal solution
    ihit = which.max(abs(uhat))  # Hitting coordinate
    hit = abs(uhat[ihit])        # Critical lambda
    s = sign(uhat[ihit])         # Sign
    
    if (verbose) {
      cat(sprintf("1. lambda=%.3f, adding coordinate %i, |B|=%i...",
                  hit,ihit,1))
    }
    
    # Now iteratively find the new dual solution, and
    # the next critical point
    
    # Things to keep track of, and return at the end
    buf = min(maxsteps,1000)
    lams = numeric(buf)        # Critical lambdas
    h = logical(buf)           # Hit or leave?
    df = numeric(buf)          # Degrees of freedom
    
    lams[1] = hit
    h[1] = TRUE
    df[1] = 0
    
    u = matrix(0,m,buf)      # Dual solutions
    beta = matrix(0,p,buf)   # Primal solutions
    u[,1] = uhat
    beta[,1] = betahat
    
    # Special interior set over nodes
    I0 = rep(TRUE,numnodes)
    
    # Update the graph if we need to, otherwise
    # update the special interior set
    if (ihit <= numedges) {
      ed = which(D[ihit,]!=0)
      gr[ed[1],ed[2]] = 0             # Delete edge
      newcl = subcomponent(gr,ed[1])  # New cluster
      oldcl = which(i==i[ed[1]])      # Old cluster
      # If these two clusters aren't the same, update
      # the memberships
      if (length(newcl)!=length(oldcl) || any(sort(newcl)!=sort(oldcl))) {
        i[newcl] = q+1
        q = q+1
      }
    }
    else {
      I0[ihit-numedges] = FALSE
    }
    
    # Other things to keep track of, but not return
    r = 1                      # Size of boundary set
    B = ihit                   # Boundary set
    I = Seq(1,m)[-ihit]        # Interior set
    D1 = D[-ihit,,drop=FALSE]  # Matrix D[I,]
    D2 = D[ihit,,drop=FALSE]   # Matrix D[B,]
    k = 2                      # What step are we at?
  }
  
  # If iterating already started path
  else {
    # Grab variables from outer object
    lambda = NULL
    for (j in 1:length(object)) {
      if (names(object)[j] != "pathobjs") {
        assign(names(object)[j], object[[j]])
      }
    }
    
    # Trick: save y,X from outer object
    y0 = y
    X0 = X
    
    # Grab variables from inner object
    for (j in 1:length(object$pathobjs)) {
      assign(names(object$pathobjs)[j], object$pathobjs[[j]])
    }
    lams = lambda
    
    # In the case of a ridge penalty, modify X
    if (eps>0) X = rbind(X,diag(sqrt(eps),p))
  }
  
  tryCatch({
    while (k<=maxsteps && lams[k-1]>=minlam) {
      ##########
      # Check if we've reached the end of the buffer
      if (k > length(lams)) {
        buf = length(lams)
        lams = c(lams,numeric(buf))
        h = c(h,logical(buf))
        df = c(df,numeric(buf))
        u = cbind(u,matrix(0,m,buf))
        beta = cbind(beta,matrix(0,p,buf))
      }
      
      ##########
      Ds = as.numeric(t(D2)%*%s)
      
      # Precomputation for the hitting times: first we project
      # y and Ds onto the row space of D1*X^+
      A = matrix(0,n,q)
      z = matrix(0,q,2)
      nz = rep(FALSE,q)
      
      # For efficiency, don't loop over singletons
      tab = tabulate(i)
      oo = which(tab[i]==1)
      oo2 = oo[!I0[oo]]
      if (length(oo2)>0) {
        j = i[oo2]
        A[,j] = X[,oo2,drop=FALSE]
        z[j,1] = xy[oo2]
        z[j,2] = Ds[oo2]
        nz[j] = TRUE
      }
      
      # Now consider all groups with at least two elements
      grps = which(tab>1)
      for (j in grps) {
        oo = which(i==j)
        if (all(!I0[oo])) {
          A[,j] = rowMeans(X[,oo,drop=FALSE])
          z[j,1] = mean(xy[oo])
          z[j,2] = mean(Ds[oo])
          nz[j] = TRUE
        }
      }
      
      nzq = sum(nz)
      e = matrix(0,q,2)
      if (nzq>0) {
        R = qr.R(qr(A[,nz]))
        e[nz,] = backsolve(R,forwardsolve(R,z[nz,,drop=FALSE],upper.tri=TRUE,transpose=TRUE))
        # Note: using a QR here is preferable than simply calling
        # e[nz,] = solve(crossprod(A[,nz]),z[nz,,drop=FALSE]), for
        # numerical stablity. Plus, it's not really any slower
      }
      ea = e[,1]
      eb = e[,2]
      ga = xy-t(X)%*%(A%*%ea)
      gb = Ds-t(X)%*%(A%*%eb)
      
      # If the interior is empty, then nothing will hit
      if (r==m) {
        fa = ea[i]
        fb = eb[i]
        hit = 0
      }
      
      # Otherwise, find the next hitting time
      else {
        # Here we perform our usual fused lasso solve but
        # with ga in place of y and gb in place of Ds
        xa = xb = numeric(p)
        fa = fb = numeric(p)
        
        # For efficiency, don't loop over singletons
        oo = which(tab[i]==1)
        fa[oo] = ea[i][oo]
        fb[oo] = eb[i][oo]
        oo1 = oo[I0[oo]]
        if (length(oo1)>0) {
          Ldiag = diag(crossprod(Matrix(D1[,oo1],sparse=TRUE)))
          xa[oo1] = ga[oo1]/Ldiag
          xb[oo1] = gb[oo1]/Ldiag
        }
        
        # Now all groups with at least two elements
        oi = order(i)
        cs = cumsum(tab)
        grps = which(tab>1)
        for (j in grps) {
          oo = oi[Seq(cs[j]-tab[j]+1,cs[j])]
          fa[oo] = ea[j]/length(oo)
          fb[oo] = eb[j]/length(oo)
          gaj = ga[oo]
          gbj = gb[oo]
          
          if (any(I0[oo])) {
            Lj = crossprod(Matrix(D1[,oo],sparse=TRUE))
            xa[oo] = as.numeric(solve(Lj,gaj))
            xb[oo] = as.numeric(solve(Lj,gbj))
          }
          else {
            Lj = crossprod(Matrix(D1[,oo[-1]],sparse=TRUE))
            xa[oo][-1] = as.numeric(solve(Lj,(gaj-mean(gaj))[-1]))
            xb[oo][-1] = as.numeric(solve(Lj,(gbj-mean(gbj))[-1]))
          }
        }
        
        a = as.numeric(D1%*%xa)
        b = as.numeric(D1%*%xb)
        shits = Sign(a)
        hits = a/(b+shits)
        
        # Make sure none of the hitting times are larger
        # than the current lambda (precision issue)
        hits[hits>lams[k-1]+btol] = 0
        hits[hits>lams[k-1]] = lams[k-1]
        
        ihit = which.max(hits)
        hit = hits[ihit]
        shit = shits[ihit]
      }
      
      ##########
      # If nothing is on the boundary, then nothing will leave
      # Also, skip this if we are in "approx" mode
      if (r==0 || approx) {
        leave = 0
      }
      
      # Otherwise, find the next leaving time
      else {
        c = as.numeric(s*(D2%*%fa))
        d = as.numeric(s*(D2%*%fb))
        leaves = c/d
        
        # c must be negative
        leaves[c>=0] = 0
        
        # Make sure none of the leaving times are larger
        # than the current lambda (precision issue)
        leaves[leaves>lams[k-1]+btol] = 0
        leaves[leaves>lams[k-1]] = lams[k-1]
        
        ileave = which.max(leaves)
        leave = leaves[ileave]
      }
      
      ##########
      # Stop if the next critical point is negative
      if (hit<=0 && leave<=0) break
      
      # If a hitting time comes next
      if (hit > leave) {
        # Record the critical lambda and properties
        lams[k] = hit
        h[k] = TRUE
        df[k] = nzq
        uhat = numeric(m)
        uhat[B] = hit*s
        uhat[I] = a-hit*b
        betahat = fa-hit*fb
        
        # Update our graph if we need to, otherwise
        # update the special interior set
        if (I[ihit] <= numedges) {
          ed = which(D1[ihit,]!=0)
          gr[ed[1],ed[2]] = 0             # Delete edge
          newcl = subcomponent(gr,ed[1])  # New cluster
          oldcl = which(i==i[ed[1]])      # Old cluster
          # If these two clusters aren't the same, update
          # the memberships
          if (length(newcl)!=length(oldcl) || any(sort(newcl)!=sort(oldcl))) {
            i[newcl] = q+1
            q = q+1
          }
        }
        else {
          I0[I[ihit]-numedges] = FALSE
        }
        
        # Update all other variables
        r = r+1
        B = c(B,I[ihit])
        I = I[-ihit]
        s = c(s,shit)
        D2 = rbind(D2,D1[ihit,])
        D1 = D1[-ihit,,drop=FALSE]
        
        if (verbose) {
          cat(sprintf("\n%i. lambda=%.3f, adding coordinate %i, |B|=%i...",
                      k,hit,B[r],r))
        }
      }
      
      # Otherwise a leaving time comes next
      else {
        # Record the critical lambda and properties
        lams[k] = leave
        h[k] = FALSE
        df[k] = nzq
        uhat = numeric(m)
        uhat[B] = leave*s
        uhat[I] = a-leave*b
        betahat = fa-leave*fb
        
        # Update our graph if we need to, otherwise
        # update the special interior set
        if (B[ileave] <= numedges) {
          ed = which(D2[ileave,]!=0)
          gr[ed[1],ed[2]] = 1             # Add edge
          newcl = subcomponent(gr,ed[1])  # New cluster
          oldcl = which(i==i[ed[1]])      # Old cluster
          # If these two clusters aren't the same, update
          # the memberships
          if (length(newcl)!=length(oldcl) || !all(sort(newcl)==sort(oldcl))) {
            newno = i[ed[2]]
            oldno = i[ed[1]]
            i[oldcl] = newno
            i[i>oldno] = i[i>oldno]-1
            q = q-1
          }
        }
        else {
          I0[B[ileave]-numedges] = TRUE
        }
        
        # Update all other variables
        r = r-1
        I = c(I,B[ileave])
        B = B[-ileave]
        s = s[-ileave]
        D1 = rbind(D1,D2[ileave,])
        D2 = D2[-ileave,,drop=FALSE]
        
        if (verbose) {
          cat(sprintf("\n%i. lambda=%.3f, deleting coordinate %i, |B|=%i...",
                      k,leave,I[m-r],r))
        }
      }
      
      u[,k] = uhat
      beta[,k] = betahat
      
      # Step counter
      k = k+1
    }
  }, error = function(err) {
    err$message = paste(err$message,"\n(Path computation has been terminated;",
                        " partial path is being returned.)",sep="")
    warning(err)})
  
  # Trim
  lams = lams[Seq(1,k-1)]
  h = h[Seq(1,k-1)]
  df = df[Seq(1,k-1)]
  u = u[,Seq(1,k-1),drop=FALSE]
  beta = beta[,Seq(1,k-1),drop=FALSE]
  
  # If we reached the maximum number of steps
  if (k>maxsteps) {
    if (verbose) {
      cat(sprintf("\nReached the max number of steps (%i),",maxsteps))
      cat(" skipping the rest of the path.")
    }
    completepath = FALSE
  }
  
  # If we reached the minimum lambda
  else if (lams[k-1]<minlam) {
    if (verbose) {
      cat(sprintf("\nReached the min lambda (%.3f),",minlam))
      cat(" skipping the rest of the path.")
    }
    completepath = FALSE
  }
  
  # Otherwise, note that we completed the path
  else completepath = TRUE
  
  # The least squares solution (lambda=0)
  bls = NULL
  if (completepath) bls = fa
  if (verbose) cat("\n")
  
  # Save needed elements for continuing the path
  pathobjs = list(type="fused.l1.x" ,r=r, B=B, I=I, Q1=NA, approx=approx,
                  Q2=NA, k=k, df=df, D1=D1, D2=D2, Ds=Ds, ihit=ihit, m=m, n=n, p=p, q=q,
                  h=h, q0=NA, rtol=rtol, btol=btol, eps=eps, s=s, y=y, gr=gr, i=i,
                  numedges=numedges, I0=I0, xy=xy)
  
  colnames(u) = as.character(round(lams,3))
  colnames(beta) = as.character(round(lams,3))
  return(list(lambda=lams,beta=beta,fit=X0%*%beta,u=u,hit=h,df=df,y=y0,X=X0,
              completepath=completepath,bls=bls,gamma=gamma,pathobjs=pathobjs))
}

Seq <- function(a, b, ...) {
  if (a <= b)
    return(seq(a, b, ...))
  else
    return(numeric(0))
}

Sign <- function(x) {
  return(-1 + 2 * (x >= 0))
}


# Construction of spatial weight matrix
Construct_SpatialWeightMatrix <- function(bandwidth, coordinate) {
  D <- rdist(coordinate) # Distance matrix
  C <- exp(-(D / bandwidth) ^ 2) # Squared exponential kernel 
  return(list(as.matrix(D), as.data.frame(as.matrix(C))))
}


# Approximate calculation of eigenvectors by Murakami and Griffith (2019)
eigen_approx<- function(coords, bandwidth, knots, threshold_evec){
  n <- nrow(coords)
  enum <- knots # Number of knots
  h <- bandwidth
  coordk <- kmeans( coords, centers = enum, iter.max = 50 )$centers # Center coords of knots created by k-means
  D <- rdist( coordk ) # Distance matrix of knots
  C	<- exp( -( D / h) ^ 2 ) # Spatial weight matrix of knots
  C <- as.data.frame(as.matrix(C))
  diag(C) <- 0
  Cmean	  <- apply( C, 1, mean )
  MCM	    <- t( C - Cmean ) - Cmean + mean( Cmean )
  eigenC	<- eigen( MCM ) 
  sf0	    <- eigenC$vectors
  ev0	    <- eigenC$values
  L <- nrow(coordk)
  
  Dnl <- cdist( coords, coordk ) # Distance matrix between data and knots
  Cnl <- exp( -( Dnl / h) ^ 2 )  # Spatial weight matrix between data and knots
  one<-matrix(1, ncol=1, nrow=n) # n*1 vector of 1s
  one_l<-matrix(1, ncol=1, nrow=L) # L*1 vector of 1s
  identity<-diag(ncol=L, nrow=L) #L*L identity matrix
  eigen_lam<-diag(x=ev0, ncol=L, nrow=L) #??_L
  
  C_l<-as.matrix(C)
  
  block1<-(t(one_l)%*%C_l)/L 
  block2<- one %x% block1 
  block3<-Cnl-block2
  block4<-(identity+eigen_lam)
  block5<-block4
  for(i in 1:nrow(block5)){
    block5[i,i]<-1/block4[i,i]  
  }
  approx_eigen2<-block3%*%sf0%*%block5 
  
  lambda<-((L+n)/L)*(identity+eigen_lam)-identity
  
  evalue2<-diag(lambda)
  evalue2<-as.matrix(as.data.frame(evalue2))
  evalue3<-evalue2
  hiritsu<-matrix(0,ncol=1,nrow=nrow(evalue2))
  for(t in 1:nrow(hiritsu)){
    hiritsu[t,]<-evalue3[t,]/evalue3[1,]  
  }
  evalue3<-as.data.frame(cbind(evalue3,hiritsu))
  colnames(evalue3)<-c("V1","V2")
  evalue3<-evalue3[evalue3$V2 > threshold_evec,]
  
  approx_eigen<-approx_eigen2[,c(1:nrow(evalue3))] 
  
  # Standardize eigenvectors
  for(i in 1:length(evalue3)){
    absolute_value<-norm(approx_eigen2[,i], "2")
    approx_eigen[,i]<-approx_eigen2[,i]/absolute_value
  }
  
  return(list(approx_eigen, lambda, evalue3$V1)) 
}


# Estimation of Fused LASSO
# D: Matrix that represents the constraints on subregion dummies (L1 penalty on themselves and difference between neighboring coefficients)
fusedlasso_func <- function(y, X, X_dummy, D_subregion, steps, gamma_dummy) {
  # Set explanatory variables including subregion dummies
  X_all <- cbind(X, X_dummy)
  
  # Set L1 constraint matrix
  # Add matrix of 0 elements on the left side of D_subregion
  D_zero <- matrix(0, nrow = nrow(D_subregion), ncol = ncol(X))
  
  # Set weights gamma_dummy on L1 regularization term for coefficients (explanatory variables and subregion dummies)
  D_coef <-
    diag(gamma_dummy,
         nrow = ncol(X_all),
         ncol = ncol(X_all))
  # Set small weight on L1 regularization term for coefficient on explanatory variables
  for (i in 1:ncol(X)) {
    D_coef[i, i] <- 1e-6
  }
  
  D0 <- rbind(cbind(D_zero, D_subregion), D_coef)
  D <- as(D0, "dgCMatrix")
  
  best_result <- Search_best_model_by_OLS(y, X_all, D, D0, steps)
  coef_all <- as.data.frame(best_result[[3]])
  coef_subregion <- coef_all[c((ncol(X) + 1):nrow(coef_all)), ]
  
  return(list(
    # lambda when the best model (evaluated by BIC estimated by OLS) is estimated
    best_result[[1]],
    # BIC of OLS estimates of the best model
    best_result[[2]],
    # alpha (coefficients of subregion dummies)
    coef_subregion,
    # All coefficients
    coef_all,
    # Number of effective explanatory variables
    best_result[[4]]
  ))
}


### Search lambda that output the minimum BIC by OLS estimation
### Fusion model
ESFGLSVC_func <-
  function(y, X_ESF, X_dummy, D_adj_subregion, E, steps, gamma_eigenvector, gamma_subregion) {
    # Set explanatory variables including subregion dummies
    X_all <- cbind(X_ESF, X_dummy)
    
    # Set L1 constraint matrix
    # Add matrix of 0 elements on the left side of D_subregion_adj (info of subregions' adjacency)
    D_zero <-
      matrix(0, nrow = nrow(D_adj_subregion), ncol = ncol(X_ESF))
    
    # Set weight "gamma_eigenvector" on L1 regularization term for coefficients on explanatory * eigenvector except explanatory variables (1e-6).
    D_explanatory <-
      diag(gamma_eigenvector,
           nrow = ncol(X_all),
           ncol = ncol(X_all))
    
    # Set small weight on L1 regularization term for coefficient on explanatory variables
    n_coef <- floor(ncol(X_ESF) / (ncol(E)+1))
    for (i in 1:n_coef) {
      D_explanatory[(i - 1) * ncol(E) + 1, (i - 1) * ncol(E) + 1] <- 1e-6
    }
    
    # Set weight "gamma_subregion" on L1 regularization term for coefficients on subregion dummies.
    dum_begin <- ncol(X_ESF) + 1
    for (i in dum_begin:ncol(X_all)) {
      D_explanatory[i, i] <- gamma_subregion
    }
    
    D0 <- rbind(cbind(D_zero, D_adj_subregion), D_explanatory)
    D <- as(D0, "dgCMatrix")
    
    best_result <- Search_best_model_by_OLS(y, X_all, D, D0, steps)
    coef_all <- as.data.frame(best_result[[3]])
    
    coef_svc <- matrix(0, ncol = n_coef, nrow = nrow(E))
    
    # Number of parameters that correspond to one spatially varying coef.
    col_group <- ncol(E) + 1
    
    for (i in 1:n_coef) {
      # Non-spatially varying coef is located at the first column. 
      coef_constant <-
        matrix(coef_all[1 + (i - 1) * col_group, ], nrow = nrow(E), ncol = 1)
      
      # Coef of X*eigenvectors are located from second to last columns.
      coef_eigenvector <-
        coef_all[c((2 + (i - 1) * col_group):(i * col_group)), ]
      coef_eigenvector <- as.matrix(coef_eigenvector, ncol = 1)
      
      coef_svc[, i] <- coef_constant + E %*% coef_eigenvector
    }
    
    # Subregion-based coef.
    coef_subregion <-
      as.data.frame(coef_all[c((nrow(coef_all) - ncol(D_adj_subregion) + 1):nrow(coef_all)), ])
    
    return(list(
      # lambda when the best model (evaluated by BIC estimated by OLS) is estimated
      best_result[[1]],
      # BIC of OLS estimates of the best model
      best_result[[2]],
      # Estimated spatially varying coefficients of each explanatory variable at each location
      coef_svc,
      # Estimated coefficients of subregion dummies
      coef_subregion,
      # All estimated coefficients
      coef_all,
      # Number of effective explanatory variables
      best_result[[4]]
    ))
  }


### Calculate coefficient of determination (R-square and adjusted R-square)
calc_R2 <- function(y, residuals, df) {
  mean_y <- matrix(mean(y), ncol = 1, nrow = nrow(y))
  n <- nrow(y)
  sq_difference <- (y - mean_y) ^ 2
  denominator <- sum(sq_difference)
  numerator <- sum(residuals * residuals)
  return(list(1 - numerator / denominator, 1 - (numerator / (n - df - 1)) / (denominator / (n - 1))))
}


## Calculate Moran's I
calc_moran <- function(data, weightmat) {
  data <- as.matrix(data, ncol = 1)
  n <- nrow(data)
  mean_mat <- matrix(mean(data), nrow = n, ncol = 1)
  Vec <- data - mean_mat
  moransI <-
    n * (t(Vec) %*% (weightmat) %*% (Vec)) / (sum(weightmat) * (t(Vec) %*% (Vec)))
  return(moransI)
}


## Set reference subregion: Search subregion where sum of squares of residuals are the minimum
##                          and set it as reference subregion.
## (order of subregions recorded in subregion_dummy and subregion_id may not be the same)
# sq_resid_point: Square of residual at each points.
# subregion_id: Subregion where each point is located in is recorded.
# subregion_dummy: Matrix of subregion dummies. 
# subregion_list: list of subregions.
set_reference_subregion_order <-
  function(sq_residuals, subregion_id, subregion_dummy, subregion_list) {
    sum_sq_resid_subregion <- n_data_subregion <- tmp_n_data_subregion <- matrix(0, nrow = nrow(subregion_list), ncol = 1)
    tmp_sum_sq_resid_subregion <- aggregate(x = sq_residuals, by = list(subregion_id), sum)

    for (i in 1:nrow(subregion_list)) {
      tmp_n_data_subregion[i, ] <- sum(subregion_dummy[, i]) # count points in each subregion
      
      for (j in 1:nrow(subregion_list)) {
        if (subregion_list[i, ] == tmp_sum_sq_resid_subregion[j, 1]) {
          sum_sq_resid_subregion[i, ] <- tmp_sum_sq_resid_subregion[j, 2]
          break
        }
      }
    }
    
    tmp_subregion_dummy_name <- as.data.frame(colnames(subregion_dummy))
    tmp_n_data_subregion <- cbind(tmp_subregion_dummy_name, tmp_n_data_subregion)
    
    for (i in 1:nrow(subregion_list)){
      for (k in 1:nrow(subregion_list)){
        if (subregion_list[i,] == tmp_n_data_subregion[k, 1]){
          n_data_subregion[i,] <- tmp_n_data_subregion[k, 2]
          break
        }
      }
    }
    
    RMSE_subregion <- sqrt(sum_sq_resid_subregion / n_data_subregion)
    rownames(RMSE_subregion)<-subregion_list[,1]

    for(i in 1:nrow(RMSE_subregion)){
      if(n_data_subregion[i,] < 10){
        RMSE_subregion[i,] <- 9999
      }
    }    

    min_RMSE_subregion <- which.min(RMSE_subregion)
    min_RMSE_subregion_name <- as.character(subregion_list[min_RMSE_subregion,])
    new_subregion_dummy <- select(subregion_dummy, -all_of(min_RMSE_subregion_name))
    
    return(list(min_RMSE_subregion_name, new_subregion_dummy))
  }


#### ESF-SVC estimation ####
# X_ESF includes terms (explanatory variables * eigenvectors)
ESFSVC_func <-
  function(y, X_ESF, E, steps) {
    # Set weight 1.0 on L1 regularization term for coefficients on (explanatory variables * eigenvectors) .
    D0 <- diag(1.0,
               nrow = ncol(X_ESF),
               ncol = ncol(X_ESF))

    # Number of spatially varying coefficients
    n_coef <- floor(ncol(X_ESF) / (ncol(E)+1))
    
    # Set small weight on L1 regularization term for coefficient on explanatory variables.
    for (i in 1:n_coef) {
      D0[(i - 1) * ncol(E) + 1, (i - 1) * ncol(E) + 1] <- 1e-6
    }
    
    D <- as(D0, "dgCMatrix")
    
    # Set candidate models by lasso and select the best model based on BIC by OLS.
    best_result <- Search_best_model_by_OLS(y, X_ESF, D, D0, steps)
    coef_all <- as.data.frame(best_result[[3]])
    
    coef_svc <- matrix(0, nrow = nrow(E), ncol = n_coef)
    
    # Number of parameters that correspond to one spatially varying coef.
    col_group <- ncol(E) + 1
    
    for (i in 1:n_coef) {
      # Non-spatially varying coef is located at the first column. 
      coef_constant <-
        matrix(coef_all[1 + (i - 1) * col_group,], nrow = nrow(E), ncol = 1)
      
      # Coef of X*eigenvectors are located from second to last columns.
      coef_eigenvector <-
        coef_all[c((2 + (i - 1) * col_group):(i * col_group)),]
      coef_eigenvector <- as.matrix(coef_eigenvector, ncol = 1)
      
      coef_svc[, i] <- coef_constant + E %*% coef_eigenvector
    }
    
    return(list(
      # lambda when the best model (evaluated by BIC estimated by OLS) is estimated
      best_result[[1]],
      # Minimum BIC
      best_result[[2]],
      # Estimated spatially varying coefficients of each explanatory variable at each location
      coef_svc,
      # Estimated coefficients
      coef_all,
      # Number of effective explanatory variables
      best_result[[4]]
    ))
  }


### Search the best estimation result based on BIC by OLS estimation.
Search_best_model_by_OLS <- 
  function(y, X, D, D0, steps){
    # y: Dependent variables.
    # X: Explanatory variables (including variables calculated by eigenvectors).
    # D: Settings of weights on L1 regularization by sparse matrix (dgCMatrix).
    # D0: Settings of weights on L1 regularization.
    
    flsa = dualpathFusedL1X(
      y = y,
      X = X,
      D = D,
      D0 = D0,
      gamma = 1.0, # gamma is note effective in the function. The gamma should be represented by D and D0.
      approx = FALSE,
      maxsteps = steps,
      minlam = 0.001,
      rtol = 1e-7,
      btol = 1e-7,
      eps = 1e-4,
      verbose = FALSE,
      object = NULL
    )
    flsa$call = match.call()
    class(flsa) = c("fusedlasso", "genlasso", "list")
    
    summary.genlasso <- function(object) {
      n = length(object$y)
      p = nrow(object$beta)
      df = object$df
      y2 <- matrix(0, nrow = n, ncol = ncol(object$fit))
      object$y2 <- as.data.frame(y2)
      for (i in 1:ncol(object$y2)) {
        object$y2[, i] = y
      }
      
      rss = colSums((object$y2 - object$fit) ^ 2)
      
      mat = cbind(object$df, object$lambda, rss)
      rownames(mat) = rep("", nrow(mat))
      colnames(mat) = c("df", "lambda", "rss")
      
      class(mat) = c("summary.genlasso", "matrix")
      return(mat)
    }
    
    summary <- as.data.frame(summary.genlasso(object = flsa))
    group_df <- dplyr::group_by(summary, df)
    group_df <-
      dplyr::summarise(group_df,
                       min_lambda = min(lambda),
                       min_rss = min(rss))
    n <- nrow(y)
    bic <-
      dplyr::mutate(group_df, BIC = n * (log(2 * pi * min_rss / n) + 1) + df *
                      log(n)) # BIC calculated from estimates by lasso
    
    # Prepare a column to input BIC by OLS and number of effective coef, and ID.
    OLS_bic <- bic$BIC
    effective_coef <- bic$BIC
    bic_id <- matrix(1:nrow(bic), nrow = nrow(bic), ncol = 1)
    bic <- cbind(bic, OLS_bic, effective_coef, bic_id)
    
    temp_OLS_bic <- matrix(1e+100, nrow = nrow(bic), ncol = 1)
    temp_effective_coef <- matrix(0, nrow = nrow(bic), ncol = 1)
    
    # Recalculate BIC by OLS with explanatory variables whose coefficients are non-zero by fused lasso estimation
    # (starting from the model whose number of parameters is two)
    estimated_coef <- as.data.frame(matrix(0, nrow=ncol(X), ncol=1))
    
    for (j in 2:nrow(bic)) {
      temp_X <- X
      lambda <- bic[j, ]$min_lambda
      coef_lasso <- as.data.frame(as.matrix(coefficients(flsa, lambda = lambda)$beta, ncol = 1))
      
      # Set ID for coefficients
      id <- matrix(1:nrow(coef_lasso))
      coef_lasso <- cbind(id, coef_lasso)
      
      # Set ID for columns of explanatory matrix temp_X (=X_lasso)
      colnames(temp_X) <- id
      temp_X <- as.data.frame(temp_X)
      colnames(coef_lasso) <- c("id", "param")
      
      # remove zero-coefficients
      coef_non_zero <- dplyr::filter(coef_lasso, param != 0)
      
      # remove explanatory variables whose estimated coefficients are zero
      nonzeroid <- unique(coef_non_zero$id)
      X_non_zero <- temp_X[, nonzeroid]
      Trans_non_zero <- diag(nrow = ncol(X), ncol = ncol(X))[, nonzeroid]
      
      # id2: ID for columuns of X_non_zero and coef_non_zero
      id2 <- matrix(1:nrow(coef_non_zero))
      coef_non_zero <- cbind(id2, coef_non_zero)
      
      # Search coefficients whose values are the same (coef of adjacent subregion dummies)
      overlapped_id <- coef_non_zero %>% group_by(param) %>% filter(n() > 1)
      
      # If there are no coefficients with the same values
      if (nrow(overlapped_id) == 0){
        Trans_no_overlap <- Trans_non_zero
      }
      else{
        overlap_list <- as.matrix(unique(overlapped_id$param), ncol = 1)
        
        # Aggregate coefficients that have same values
        for (k in 1:nrow(overlap_list)) {
          # Record coefficients who have same value with overlap_list[k]
          overlapped_coef <-
            dplyr::filter(overlapped_id, param == overlap_list[k, ])
          
          # Extract columns of transformation matrix whose coefficient values are in overlap_list[k]
          # and calculate the sum of columns.
          Trans_col_non_zero_overlap <- Trans_non_zero[, overlapped_coef$id2]
          Trans_non_zero <- cbind(Trans_non_zero, rowSums(Trans_col_non_zero_overlap))
        }
        # remove explanatory variables of overlapped coef
        Trans_no_overlap <- Trans_non_zero[, -overlapped_id$id2]
      }
      
      Trans_no_overlap <- as.matrix(Trans_no_overlap)
      temp_X <- as.matrix(temp_X)
      X_no_overlap <- temp_X %*% Trans_no_overlap
      
      # Output BIC by OLS estimation
      OLS <- lm(y ~ X_no_overlap - 1)
      
      # Record BIC and # of effective coefficients
      temp_OLS_bic[j, ] <- BIC(OLS)
      temp_effective_coef[j, ] <- ncol(X_no_overlap)
      
      coef <- as.data.frame(Trans_no_overlap %*% as.matrix(OLS$coefficients, ncol = 1))
      estimated_coef <- cbind(estimated_coef, coef)
    }
    bic[, 5] <- temp_OLS_bic
    bic[, 6] <- temp_effective_coef
    
    min_bic <- min(as.matrix(bic %>% dplyr::select(OLS_bic)))
    best_lambda <- as.numeric(bic %>% dplyr::filter(OLS_bic == min_bic) %>% dplyr::select(min_lambda) %>% head(1))
    bic_id <- as.numeric(bic %>% dplyr::filter(OLS_bic == min_bic) %>% dplyr::select(bic_id) %>% head(1))
    
    best_coef <- as.matrix(estimated_coef[ ,bic_id])
    best_N_effective_coef <- bic[bic_id, 6]
    
    return(list(
      # lambda that outputs the best model (evaluated by BIC estimated by OLS)
      best_lambda,
      # Minimum BIC by OLS estimation
      min_bic,
      # Estimated coefficients of the best model
      best_coef,
      # Number of effective explanatory variables
      best_N_effective_coef
    ))  
  }


##################################
############## Main ##############
##################################

#### Hyperparameter settings #### 
# Fusion model
lambda_list <- matrix(c(0.1,0.1,0.1,1,0.1,10,
                        1,0.1,1,1,1,10,
                        10,0.1,10,1,10,10), nrow=2)
rownames(lambda_list) <- c("lambda1","lambda2")

# Fused lasso
fusedlasso_lambda <- matrix(c(1/10, 1,  10), nrow=3)

#### Bandwidth setting by meters ####
bandwidth_setting <- matrix(c(1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000), nrow=9, ncol=1)


#### Input data ####
dataset <- as.data.frame(read.csv("dataset_sample.csv"))

# Explained variable (ln (monthly rent / property size))
y <- as.matrix(dataset$rent_sq_meter)

# Compose the matrix of explanatory variables (X)
# First row: A vector of 1s.
# Second to last rows: Explanatory variables (logarithm of each variable are scaled)
X <- cbind(matrix(1, nrow=nrow(dataset), ncol=1), dataset$time_to_train_service, dataset$time_to_CBD, dataset$building_age, dataset$property_size, dataset$floor_level)
colnames(X) <- c("intercept", "time_to_train_service", "time_to_CBD", "building_age","property_size", "floor_level")

# Coordinates of data
coordinateXY <- cbind(dataset$X, dataset$Y)  # Plane rectangular coordinate of Japan
colnames(coordinateXY)<-c("X", "Y")

coordinateEN <- cbind(dataset$longitude, dataset$latitude)  # Longitude and latitude
colnames(coordinateEN)<-c("E", "N")


# Analyze by different bandwidth setting
for (b in 1:nrow(bandwidth_setting)) {

  ### Construction of spatial weight matrix for Moran's I calculation
  weightm <- as.matrix(as.data.frame(Construct_SpatialWeightMatrix(bandwidth=bandwidth_setting[b], coordinate = coordinateXY)[2]))
  
  
  #### Calculation of eigenvectors ####
  # Approximate calculation of eigenvectors
  Estimate_eigenvector <- eigen_approx(coords=coordinateXY, bandwidth=bandwidth_setting[b], knots=250, threshold_evec=0.25)
  
  # Output eigenvectors
  E <- as.matrix(as.data.frame(Estimate_eigenvector[1]))
  write.csv(cbind(E, coordinateXY), paste("bandwidth=", bandwidth_setting[b], "_Eigenvectors.csv"), row.names = FALSE)
  
  # Output eigenvalues
  evalue <- diag(as.matrix(as.data.frame(Estimate_eigenvector[2])))
  write.csv(evalue, paste("bandwidth=", bandwidth_setting[b], "_Eigenvalues.csv"), row.names = FALSE)
  
  
  #### Estimation by ESF-SVC ####
  # Build a matrix of explanatory variables for ESF-SVC estimation (X_ESF)
  X_ESF <- matrix(0, nrow=nrow(E), 0)
  for(k in 1:ncol(X)){
    Xeigen <- matrix(0, nrow=nrow(E), ncol=ncol(E))
    for(i in 1:ncol(E)){
      Xeigen[,i] <- X[,k] * E[,i]
    }
    X_ESF <- cbind(X_ESF, X[ ,k], Xeigen)
  }
  
  # Estimation
  ESFSVC <- ESFSVC_func(y=y, X_ESF=X_ESF, E=E, steps=1000)
  
  # Output the estimation results of ESF-SVC
  ESFSVC_coef_by_location <- as.data.frame(ESFSVC[3])
  colnames(ESFSVC_coef_by_location) <- colnames(X)
  write.csv(cbind(coordinateEN, as.data.frame(as.data.frame(ESFSVC_coef_by_location))), paste("bandwidth=", bandwidth_setting[b], "_ESFSVC_coef_by_location.csv"), row.names = FALSE)
  
  ESFSVC_coef <- as.matrix(as.data.frame(ESFSVC[4]))
  write.csv(ESFSVC_coef, paste("bandwidth=", bandwidth_setting[b], "_ESFSVC_allcoefs.csv"))
  
  ESFSVC_residuals <- y - X_ESF %*% ESFSVC_coef
  write.csv(cbind(coordinateEN, as.data.frame(ESFSVC_residuals)), paste("bandwidth=", bandwidth_setting[b], "_ESFSVC_residuals.csv"), row.names = FALSE)
  
  ESFSVC_moran_residual <- calc_moran(data = ESFSVC_residuals, weightmat = weightm)
  ESFSVC_R2 <- calc_R2(y=y, residuals=ESFSVC_residuals, df=ncol(X_ESF))
  ESFSVC_stats <- as.data.frame(c(bandwidth_setting[b], ESFSVC[1], ESFSVC[2], ESFSVC_R2, ESFSVC_moran_residual)) # lambda, BIC, R2, adj-R2, Moran's I
  colnames(ESFSVC_stats) <- c("bandwidth", "lambda", "BIC", "R2", "Adjusted R2","Moran's I of residuals")
  write.csv(ESFSVC_stats, paste("bandwidth=", bandwidth_setting[b], "_ESFSVC_stats.csv"), row.names = FALSE)
  
 
  #### Settings of subregion dummies. (Reference subregion, matrix of subregion dummies, adjacency of subregions)  
  # Input a subregion dummy matrix
  subregion_dummy <- read.csv("subregion_dummy_sample.csv")
  
  # Subregion name of each data
  subregion_id <- dataset$neighborhood
  
  # List of subregion names
  subregion_list <- as.data.frame(unique(subregion_id))
  
  # Set reference subregion where mean of sum of squares of residuals by ESFSVC is the minimum
  sq_residuals <- ESFSVC_residuals * ESFSVC_residuals
  set_ref_subregion <- set_reference_subregion_order(sq_residuals=sq_residuals, subregion_id = subregion_id, subregion_dummy =subregion_dummy, subregion_list=subregion_list)

  # Name of reference subregion
  reference_subregion <- set_ref_subregion[[1]]
  write.csv(reference_subregion, paste("bandwidth=", bandwidth_setting[b], "_reference_subregion.csv"))
  
  # Subregion dummy matrix without the row of reference subregion
  subregion_dummy_without_reference <- as.matrix(set_ref_subregion[[2]])
  
  # Input an adjacency matrix of subregions
  AdjMatrix <- read.csv("adjacency_matrix_sample.csv", row.names = 1)
  
  # Remove a row and column of reference subregion
  AdjMatrix_wo_ref <- AdjMatrix[!rownames(AdjMatrix)==reference_subregion, !colnames(AdjMatrix)==reference_subregion]
  AdjMatrix_wo_ref<-as.matrix(AdjMatrix_wo_ref)
  colnames(AdjMatrix_wo_ref)<-NULL
  rownames(AdjMatrix_wo_ref)<-NULL
  
  
  #### Construct sparse adjacency matrix of subregions (D) ####
  AdjGraph <- graph.adjacency(AdjMatrix_wo_ref, mode="upper")
  D <- as.matrix(getDgSparse(AdjGraph))
  colnames(D) <- colnames(AdjMatrix_wo_ref)
  
  
  #### Estimation by Fusion model ####
  # Prepare matrices to keep the results for all hyperparameter settings
  ESFGLSVC_lambda <-
    ESFGLSVC_bic <- ESFGLSVC_moran_residual <- ESFGLSVC_R2 <- ESFGLSVC_R2adj <- ESFGLSVC_df <- matrix(0, nrow = 1, ncol = ncol(lambda_list))
  ESFGLSVC_residual <- matrix(0, nrow = nrow(y), ncol = ncol(lambda_list))
  ESFGLSVC_all_coefs <- matrix(0, nrow = ncol(X_ESF)+ncol(subregion_dummy_without_reference), ncol = ncol(lambda_list))
  ESFGLSVC_dummy_estimated <-
    matrix(0, nrow = ncol(subregion_dummy_without_reference), ncol = ncol(lambda_list))
  
  # Estimate by different hyperparameter settings
  for (pp in 1:ncol(lambda_list)) {
    gamma_eigen <- lambda_list[1, pp]
    gamma_dummy <- lambda_list[2, pp]
    
    # Estimation
    Fusion <-
      ESFGLSVC_func(
        y = y,
        X_ESF = X_ESF,
        X_dummy = subregion_dummy_without_reference,
        D_adj_subregion = D,
        E = E,
        steps = 1000,
        gamma_eigenvector = gamma_eigen,
        gamma_subregion = gamma_dummy
      )
    
    # Estimated results for one setting of hyperparameters-> Record all results and return the best one
    ESFGLSVC_lambda[pp] <- as.matrix(as.data.frame(Fusion[1]))
    ESFGLSVC_bic[pp] <- as.matrix(as.data.frame(Fusion[2]))
    
    # Record estimated result and its statistics for a hyperparameter setting
    ESFGLSVC_dummy_estimated[, pp] <- as.matrix(as.data.frame(Fusion[4]))
    
    X_fusion <- cbind(X_ESF, subregion_dummy_without_reference)
    allcoefs <- as.matrix(as.data.frame(Fusion[5]))
    ESFGLSVC_all_coefs[,pp] <- allcoefs
    residual <- y - X_fusion %*% allcoefs
    ESFGLSVC_residual[, pp] <- residual
    ESFGLSVC_df[pp] <- as.numeric(Fusion[6])
    
    ESFGLSVC_moran_residual[pp] <- calc_moran(data = residual, weightmat = weightm)
    R2 <- as.matrix(as.data.frame(calc_R2(y = y, residual = residual, df =  ESFGLSVC_df[pp])))
    ESFGLSVC_R2[pp] <- R2[, 1]
    ESFGLSVC_R2adj[pp] <- R2[, 2]
  }
  
  # Output statistics of all hyperparameter settings
  ESFGLSVC_statistics <-
    rbind(
      bandwidth_setting[b], 
      lambda_list[1, ],
      lambda_list[2, ],
      ESFGLSVC_lambda,
      ESFGLSVC_bic,
      ESFGLSVC_moran_residual,
      ESFGLSVC_R2,
      ESFGLSVC_R2adj,
      ESFGLSVC_df
    )
  rownames(ESFGLSVC_statistics) <-
    c(
      "bandwidth",
      "gamma_eigenvalue",
      "gamma_dummy",
      "lambda",
      "BIC",
      "Moran's_I_residuals",
      "R2",
      "R2_adjusted",
      "Effective_coefs"
    )
  write.csv(ESFGLSVC_statistics, paste("bandwidth=", bandwidth_setting[b], "_ESFGLSVC_stats.csv"))
  
  # Search the best hyperparameter setting based on BIC
  best <- which.min(ESFGLSVC_bic)
  
  ESFGLSVC_estimates_subregion <- as.data.frame(ESFGLSVC_dummy_estimated[, best])
  rownames(ESFGLSVC_estimates_subregion) <-colnames(subregion_dummy_without_reference) 
  
  ESFGLSVC_residuals <- as.data.frame(ESFGLSVC_residual[, best])
  coef_all <- ESFGLSVC_all_coefs[,best]
  write.csv(coef_all, paste("bandwidth=", bandwidth_setting[b], "_ESFGLSVC_allcoefs.csv"))
  
  n_coef <- floor(ncol(X_ESF) / (ncol(E)+1))
  coef_svc <- matrix(0, ncol = n_coef, nrow = nrow(E))
  
  # Number of parameters that correspond to one spatially varying coef.
  col_group <- ncol(E) + 1
  
  for (i in 1:n_coef) {
    # Non-spatially varying coef is located at the first column. 
    coef_constant <-
      matrix(coef_all[1 + (i - 1) * col_group], nrow = nrow(E), ncol = 1)
    
    # Coef of X*eigenvectors are located from second to last columns.
    coef_eigenvector <-
      coef_all[c((2 + (i - 1) * col_group):(i * col_group)) ]
    coef_eigenvector <- as.matrix(coef_eigenvector, ncol = 1)
    
    coef_svc[, i] <- coef_constant + E %*% coef_eigenvector
  }
  
  ESFGLSVC_estimates_explanatory <-
    cbind(
      coordinateEN,
      coef_svc,
      subregion_dummy_without_reference %*% ESFGLSVC_dummy_estimated[, best]
    )
  colnames(ESFGLSVC_estimates_explanatory) <- cbind("E", "N", t(colnames(X)), "subregion")
  write.csv(ESFGLSVC_estimates_explanatory, paste("bandwidth=", bandwidth_setting[b], "_ESFGLSVC_coefs_by_location.csv"))
  write.csv(subregion_dummy_without_reference %*% ESFGLSVC_dummy_estimated[, best]+coef_svc[,1], paste("bandwidth=", bandwidth_setting[b], "_ESFGLSVC_Intercept+SubregionDummy.csv"))
  write.csv(ESFGLSVC_estimates_subregion, paste("bandwidth=", bandwidth_setting[b], "_ESFGLSVC_subregion_coefs.csv"))
  write.csv(ESFGLSVC_residuals, paste("bandwidth=", bandwidth_setting[b], "_ESFGLSVC_residuals.csv"))
} # End of loop for bandwidth settings


#### Estimation of ward-specific coefficients by fused lasso ####
# Input ward-specific explanatory variables.
explanatory_variables<-read.csv("ward_specific_explanatory_variables_sample.csv")
explanatory_variables<-as.matrix(explanatory_variables)

# Add intercept to compose explanatory matrix
intercept <- matrix(1, nrow=nrow(explanatory_variables), ncol=1)
X_ward <- as.matrix(cbind(intercept, explanatory_variables))
colnames(X_ward) <- c("intercept", colnames(explanatory_variables))


# Prepare matrices to keep the results for all hyperparameter settings
GL_dummy_estimated <- matrix(0, nrow = ncol(subregion_dummy_without_reference), ncol = nrow(fusedlasso_lambda))
GL_lambda <- GL_bic <- GL_moran_residual <- GL_R2 <- GL_R2adj <- GL_df <- matrix(0, nrow = nrow(fusedlasso_lambda), ncol = 1)
GL_all_coefs <- matrix(0, nrow = ncol(X_ward)+ncol(subregion_dummy_without_reference), ncol = nrow(fusedlasso_lambda))
GL_residuals <- matrix(0, nrow = nrow(y), ncol = nrow(fusedlasso_lambda))
GL_subregion <- matrix(0, nrow = ncol(subregion_dummy_without_reference), ncol = nrow(fusedlasso_lambda))


# Try different hyperparameter settings
for (rr in 1:nrow(fusedlasso_lambda)) {
  # Estimation
  FL <-
    fusedlasso_func(
      y = y,
      X = X_ward,
      X_dummy = subregion_dummy_without_reference,
      D_subregion = D,
      steps = 1000,
      gamma_dummy = fusedlasso_lambda[rr, ]
    )
  
  GL_lambda[rr] <- as.matrix(as.data.frame(FL[1]))
  GL_bic[rr] <- as.matrix(as.data.frame(FL[2]))
  GL_df[rr] <- as.numeric(FL[5])
  GL_all_coefs[, rr] <- as.matrix(as.data.frame(FL[4]))
  GL_dummy_estimated[, rr] <- as.matrix(as.data.frame(FL[3]))
  
  X_fused <- as.matrix(cbind(X_ward, subregion_dummy_without_reference))
  residuals <- y - X_fused %*% GL_all_coefs[, rr]
  GL_residuals[, rr] <- residuals
  GL_moran_residual[rr] <- calc_moran(data = residuals, weightmat = weightm)
  R2 <- as.matrix(as.data.frame(calc_R2(y = y, residual = residuals, df = GL_df[rr])))
  GL_R2[rr] <- R2[, 1]
  GL_R2adj[rr] <- R2[, 2]
  
  GL_subregion <- matrix(0, nrow = nrow(y), ncol = 1)
  GL_subregion <- subregion_dummy_without_reference %*% GL_dummy_estimated[, rr]
}

# Output statistics of all hyperparameter settings
GL_statistics <-
  cbind(
    fusedlasso_lambda,
    GL_lambda,
    GL_bic,
    GL_moran_residual,
    GL_R2,
    GL_R2adj,
    GL_df
  )

colnames(GL_statistics) <-
  c(
    "gamma_dummy",
    "lambda",
    "BIC",
    "Moran's_I_residuals",
    "R2",
    "R2_adjusted",
    "Effective_coefs"
  )
write.csv(GL_statistics,"GL_ward_stats.csv")

rownames(GL_dummy_estimated)<-colnames(subregion_dummy_without_reference)
rownames(GL_all_coefs)<-c(colnames(X_ward), colnames(subregion_dummy_without_reference))

# Search the best hyperparameter setting based on BIC
best <- which.min(GL_bic)
write.csv(GL_residuals[, best], "GL_ward_residuals.csv")
write.csv(GL_dummy_estimated[, best], "GL_ward_subregion_coefs.csv")
write.csv(GL_all_coefs[, best], "GL_ward_all_coefs.csv")

subregion_coefs <- as.matrix(as.data.frame(GL_dummy_estimated[, best]))
intercept <- as.numeric(as.data.frame(GL_all_coefs[, best][1]))
intercept_plus_subregion <- intercept + subregion_dummy_without_reference %*% subregion_coefs
write.csv(intercept_plus_subregion, "GL_ward_intercept_plus_subregion.csv")